@@ -23,6 +23,7 @@
 #include "bson.h"
 #include "bson-config.h"
 #include "bson-json.h"
+#include "bson-json-private.h"
 #include "bson-iso8601-private.h"
 
 #include "common-b64-private.h"
@@ -102,7 +103,7 @@ static const char *read_state_names[] = {FOREACH_READ_STATE (GENERATE_STRING)};
    BS (DECIMAL128)                                                   \
    BS (DBPOINTER)                                                    \
    BS (SYMBOL)                                                       \
-   BS (DBREF)
+   BS (UUID)
 
 typedef enum {
    FOREACH_BSON_STATE (BSON_STATE_ENUM)
@@ -129,8 +130,6 @@ typedef enum {
 typedef struct {
    int i;
    bson_json_frame_type_t type;
-   bool has_ref;
-   bool has_id;
    bson_t bson;
 } bson_json_stack_frame_t;
 
@@ -277,8 +276,6 @@ _noop (void)
 #define FRAME_TYPE_HAS_BSON(_type) \
    ((_type) == BSON_JSON_FRAME_SCOPE || (_type) == BSON_JSON_FRAME_DBPOINTER)
 #define STACK_HAS_BSON FRAME_TYPE_HAS_BSON (STACK_FRAME_TYPE)
-#define STACK_HAS_REF STACK_ELE (0, has_ref)
-#define STACK_HAS_ID STACK_ELE (0, has_id)
 #define STACK_PUSH(frame_type)                       \
    do {                                              \
       if (bson->n >= (STACK_MAX - 1)) {              \
@@ -307,8 +304,6 @@ _noop (void)
 #define STACK_PUSH_DOC(statement)       \
    do {                                 \
       STACK_PUSH (BSON_JSON_FRAME_DOC); \
-      STACK_HAS_REF = false;            \
-      STACK_HAS_ID = false;             \
       if (bson->n != 0) {               \
          statement;                     \
       }                                 \
@@ -392,6 +387,25 @@ _noop (void)
    }
 
 
+
+bson_json_opts_t *
+bson_json_opts_new (bson_json_mode_t mode, int32_t max_len)
+{
+   bson_json_opts_t *opts;
+
+   opts = (bson_json_opts_t *) bson_malloc (sizeof *opts);
+   opts->mode = mode;
+   opts->max_len = max_len;
+
+   return opts;
+}
+
+void
+bson_json_opts_destroy (bson_json_opts_t *opts)
+{
+   bson_free (opts);
+}
+
 static void
 _bson_json_read_set_error (bson_json_reader_t *reader, const char *fmt, ...)
    BSON_GNUC_PRINTF (2, 3);
@@ -686,12 +700,12 @@ _bson_json_read_integer (bson_json_reader_t *reader, uint64_t val, int64_t sign)
       case BSON_JSON_LF_OID:
       case BSON_JSON_LF_BINARY:
       case BSON_JSON_LF_TYPE:
+      case BSON_JSON_LF_UUID:
       case BSON_JSON_LF_UNDEFINED:
       case BSON_JSON_LF_DOUBLE:
       case BSON_JSON_LF_DECIMAL128:
       case BSON_JSON_LF_DBPOINTER:
       case BSON_JSON_LF_SYMBOL:
-      case BSON_JSON_LF_DBREF:
       default:
          _bson_json_read_set_error (reader,
                                     "Unexpected integer %s%" PRIu64
@@ -813,8 +827,29 @@ _bson_json_read_int64_or_set_error (bson_json_reader_t *reader, /* IN */
    return true;
 }
 
+static bool
+_unhexlify_uuid (const char *uuid, uint8_t *out, size_t max)
+{
+   unsigned int byte;
+   int x = 0;
+   int i = 0;
 
-/* parse a value for "base64", "subType" or legacy "$binary" or "$type" */
+   BSON_ASSERT (strlen (uuid) == 32);
+
+   while (SSCANF (&uuid[i], "%2x", &byte) == 1) {
+      if (x >= max) {
+         return false;
+      }
+
+      out[x++] = (uint8_t) byte;
+      i += 2;
+   }
+
+   return i == 32;
+}
+
+/* parse a value for "base64", "subType", legacy "$binary" or "$type", or
+ * "$uuid" */
 static void
 _bson_json_parse_binary_elem (bson_json_reader_t *reader,
                               const char *val_w_null,
@@ -874,6 +909,64 @@ _bson_json_parse_binary_elem (bson_json_reader_t *reader,
                               (int) vlen);
          }
       }
+   } else if (bs == BSON_JSON_LF_UUID) {
+      int nread = 0;
+      char uuid[33];
+
+      data->binary.has_binary = true;
+      data->binary.has_subtype = true;
+      data->binary.type = BSON_SUBTYPE_UUID;
+
+      /* Validate the UUID and extract relevant portions */
+      /* We can't use %x here as it allows +, -, and 0x prefixes */
+#ifdef _MSC_VER
+      SSCANF (val_w_null,
+              "%8c-%4c-%4c-%4c-%12c%n",
+              &uuid[0],
+              8,
+              &uuid[8],
+              4,
+              &uuid[12],
+              4,
+              &uuid[16],
+              4,
+              &uuid[20],
+              12,
+              &nread);
+#else
+      SSCANF (val_w_null,
+              "%8c-%4c-%4c-%4c-%12c%n",
+              &uuid[0],
+              &uuid[8],
+              &uuid[12],
+              &uuid[16],
+              &uuid[20],
+              &nread);
+#endif
+
+      uuid[32] = '\0';
+
+      if (nread != 36 || val_w_null[nread] != '\0') {
+         _bson_json_read_set_error (reader,
+                                    "Invalid input string \"%s\", looking for "
+                                    "a dash-separated UUID string",
+                                    val_w_null);
+
+         return;
+      }
+
+      binary_len = 16;
+      _bson_json_buf_ensure (&bson->bson_type_buf[0], (size_t) binary_len + 1);
+
+      if (!_unhexlify_uuid (
+             &uuid[0], bson->bson_type_buf[0].buf, (size_t) binary_len)) {
+         _bson_json_read_set_error (reader,
+                                    "Invalid input string \"%s\", looking for "
+                                    "a dash-separated UUID string",
+                                    val_w_null);
+      }
+
+      bson->bson_type_buf[0].len = (size_t) binary_len;
    }
 }
 
@@ -947,6 +1040,8 @@ _bson_json_read_string (bson_json_reader_t *reader, /* IN */
       case BSON_JSON_LF_BINARY:
       case BSON_JSON_LF_TYPE:
          bson->bson_type_data.binary.is_legacy = true;
+         /* FALL THROUGH */
+      case BSON_JSON_LF_UUID:
          _bson_json_parse_binary_elem (reader, val_w_null, vlen);
          break;
       case BSON_JSON_LF_INT32: {
@@ -1020,12 +1115,6 @@ _bson_json_read_string (bson_json_reader_t *reader, /* IN */
          bson_append_symbol (
             STACK_BSON_CHILD, key, (int) len, (const char *) val, (int) vlen);
          break;
-      case BSON_JSON_LF_DBREF:
-         /* the "$ref" of a {$ref: "...", $id: ... }, append normally */
-         bson_append_utf8 (
-            STACK_BSON_CHILD, key, (int) len, (const char *) val, (int) vlen);
-         bson->read_state = BSON_JSON_REGULAR;
-         break;
       case BSON_JSON_LF_SCOPE:
       case BSON_JSON_LF_TIMESTAMP_T:
       case BSON_JSON_LF_TIMESTAMP_I:
@@ -1103,7 +1192,7 @@ _is_known_key (const char *key, size_t len)
           IS_KEY ("$numberDouble") || IS_KEY ("$numberDecimal") ||
           IS_KEY ("$numberInt") || IS_KEY ("$numberLong") ||
           IS_KEY ("$numberDouble") || IS_KEY ("$numberDecimal") ||
-          IS_KEY ("$dbPointer") || IS_KEY ("$symbol"));
+          IS_KEY ("$dbPointer") || IS_KEY ("$symbol") || IS_KEY ("$uuid"));
 
 #undef IS_KEY
 
@@ -1221,6 +1310,8 @@ _bson_json_read_map_key (bson_json_reader_t *reader, /* IN */
          HANDLE_OPTION ("$binary", BSON_TYPE_BINARY, BSON_JSON_LF_BINARY)
       else if
          HANDLE_OPTION ("$type", BSON_TYPE_BINARY, BSON_JSON_LF_TYPE)
+      else if
+         HANDLE_OPTION ("$uuid", BSON_TYPE_BINARY, BSON_JSON_LF_UUID)
       else if
          HANDLE_OPTION ("$date", BSON_TYPE_DATE_TIME, BSON_JSON_LF_DATE)
       else if
@@ -1297,20 +1388,6 @@ _bson_json_read_map_key (bson_json_reader_t *reader, /* IN */
       }
    } else {
       _bson_json_save_map_key (bson, val, len);
-
-      /* in x: {$ref: "collection", $id: {$oid: "..."}, $db: "..." } */
-      if (bson->n > 0) {
-         if (!strcmp ("$ref", (const char *) val)) {
-            STACK_HAS_REF = true;
-            bson->read_state = BSON_JSON_IN_BSON_TYPE;
-            bson->bson_state = BSON_JSON_LF_DBREF;
-         } else if (!strcmp ("$id", (const char *) val)) {
-            STACK_HAS_ID = true;
-         } else if (!strcmp ("$db", (const char *) val)) {
-            bson->read_state = BSON_JSON_IN_BSON_TYPE;
-            bson->bson_state = BSON_JSON_LF_DBREF;
-         }
-      }
    }
 }
 
@@ -1369,11 +1446,6 @@ _bson_json_read_append_regex (bson_json_reader_t *reader,    /* IN */
                                     "Missing \"$regex\" after \"$options\"");
          return;
       }
-      if (!data->regex.has_options) {
-         _bson_json_read_set_error (reader,
-                                    "Missing \"$options\" after \"$regex\"");
-         return;
-      }
    } else if (!data->regex.has_pattern) {
       _bson_json_read_set_error (
          reader, "Missing \"pattern\" after \"options\" in regular expression");
@@ -1720,11 +1792,6 @@ _bson_json_read_end_map (bson_json_reader_t *reader) /* IN */
          bson->read_state = BSON_JSON_IN_BSON_TYPE_DBPOINTER_STARTMAP;
          STACK_POP_DBPOINTER;
       } else {
-         if (STACK_HAS_ID != STACK_HAS_REF) {
-            _bson_json_read_set_error (
-               reader, "%s", "DBRef object must have both $ref and $id keys");
-         }
-
          STACK_POP_DOC (
             bson_append_document_end (STACK_BSON_PARENT, STACK_BSON_CHILD));
       }
