@@ -15,11 +15,12 @@
  */
 
 
-#include "bson/bson.h"
-#include "bson/bson-config.h"
-#include "bson/bson-private.h"
-#include "bson/bson-string.h"
-#include "bson/bson-iso8601-private.h"
+#include "bson.h"
+#include "bson-config.h"
+#include "bson-private.h"
+#include "bson-json-private.h"
+#include "bson-string.h"
+#include "bson-iso8601-private.h"
 
 #include "common-b64-private.h"
 
@@ -44,13 +45,6 @@ typedef enum {
 } bson_validate_phase_t;
 
 
-typedef enum {
-   BSON_JSON_MODE_LEGACY,
-   BSON_JSON_MODE_CANONICAL,
-   BSON_JSON_MODE_RELAXED,
-} bson_json_mode_t;
-
-
 /*
  * Structures.
  */
@@ -69,6 +63,8 @@ typedef struct {
    uint32_t depth;
    bson_string_t *str;
    bson_json_mode_t mode;
+   int32_t max_len;
+   bool max_len_reached;
 } bson_json_state_t;
 
 
@@ -88,7 +84,8 @@ _bson_as_json_visit_document (const bson_iter_t *iter,
 static char *
 _bson_as_json_visit_all (const bson_t *bson,
                          size_t *length,
-                         bson_json_mode_t mode);
+                         bson_json_mode_t mode,
+                         int32_t max_len);
 
 /*
  * Globals.
@@ -105,7 +102,7 @@ static const uint8_t gZero;
  *       a malloc based buffer.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful; otherwise false indicating BSON_MAX_SIZE overflow.
  *
  * Side effects:
  *       None.
@@ -127,7 +124,7 @@ _bson_impl_inline_grow (bson_impl_inline_t *impl, /* IN */
 
    req = bson_next_power_of_two (impl->len + size);
 
-   if (req <= INT32_MAX) {
+   if (req <= BSON_MAX_SIZE) {
       data = bson_malloc (req);
 
       memcpy (data, impl->data, impl->len);
@@ -161,7 +158,7 @@ _bson_impl_inline_grow (bson_impl_inline_t *impl, /* IN */
  *       based buffers.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful; otherwise false indicating BSON_MAX_SIZE overflow.
  *
  * Side effects:
  *       None.
@@ -187,7 +184,7 @@ _bson_impl_alloc_grow (bson_impl_alloc_t *impl, /* IN */
 
    req = bson_next_power_of_two (req);
 
-   if ((req <= INT32_MAX) && impl->realloc) {
+   if ((req <= BSON_MAX_SIZE) && impl->realloc) {
       *impl->buf = impl->realloc (*impl->buf, req, impl->realloc_func_ctx);
       *impl->buflen = req;
       return true;
@@ -303,7 +300,7 @@ _bson_encode_length (bson_t *bson) /* IN */
  *
  * Returns:
  *       true if the bytes were appended successfully.
- *       false if it bson would overflow INT_MAX.
+ *       false if it bson would overflow BSON_MAX_SIZE.
  *
  * Side effects:
  *       None.
@@ -337,9 +334,16 @@ _bson_append_va (bson_t *bson,              /* IN */
 
    do {
       n_pairs--;
-      memcpy (buf, data, data_len);
-      bson->len += data_len;
-      buf += data_len;
+      /* data may be NULL if data_len is 0. memcpy is not safe to call with
+       * NULL. */
+      if (BSON_LIKELY (data_len != 0 && data != NULL)) {
+         memcpy (buf, data, data_len);
+         bson->len += data_len;
+         buf += data_len;
+      } else if (BSON_UNLIKELY (data_len != 0 && data == NULL)) {
+         /* error, user appending NULL with non-zero length. */
+         return false;
+      }
 
       if (n_pairs) {
          data_len = va_arg (args, uint32_t);
@@ -372,7 +376,7 @@ _bson_append_va (bson_t *bson,              /* IN */
  *       @first_data: First buffer.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful; otherwise false indicating BSON_MAX_SIZE overflow.
  *
  * Side effects:
  *       None.
@@ -426,7 +430,7 @@ _bson_append (bson_t *bson,              /* IN */
  *       @key_type MUST be either BSON_TYPE_DOCUMENT or BSON_TYPE_ARRAY.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful; otherwise false indicating BSON_MAX_SIZE overflow.
  *
  * Side effects:
  *       @child is initialized if true is returned.
@@ -689,7 +693,7 @@ bson_append_document_begin (bson_t *bson,    /* IN */
  *       function, if true is returned.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful; otherwise false indicating BSON_MAX_SIZE overflow.
  *
  * Side effects:
  *       @child is destroyed and invalid after calling this function.
@@ -719,7 +723,7 @@ bson_append_document_end (bson_t *bson,  /* IN */
  *       since few buffers need to be malloced.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful; otherwise false indicating BSON_MAX_SIZE overflow.
  *
  * Side effects:
  *       None.
@@ -813,7 +817,6 @@ bson_append_binary (bson_t *bson,           /* IN */
 
    BSON_ASSERT (bson);
    BSON_ASSERT (key);
-   BSON_ASSERT (binary);
 
    if (key_length < 0) {
       key_length = (int) strlen (key);
@@ -2000,7 +2003,7 @@ bson_init_static (bson_t *bson, const uint8_t *data, size_t length)
    BSON_ASSERT (bson);
    BSON_ASSERT (data);
 
-   if ((length < 5) || (length > INT_MAX)) {
+   if ((length < 5) || (length > BSON_MAX_SIZE)) {
       return false;
    }
 
@@ -2060,7 +2063,7 @@ bson_sized_new (size_t size)
    bson_impl_alloc_t *impl_a;
    bson_t *b;
 
-   BSON_ASSERT (size <= INT32_MAX);
+   BSON_ASSERT (size <= BSON_MAX_SIZE);
 
    b = bson_malloc (sizeof *b);
    impl_a = (bson_impl_alloc_t *) b;
@@ -2099,7 +2102,7 @@ bson_new_from_data (const uint8_t *data, size_t length)
 
    BSON_ASSERT (data);
 
-   if ((length < 5) || (length > INT_MAX) || data[length - 1]) {
+   if ((length < 5) || (length > BSON_MAX_SIZE) || data[length - 1]) {
       return NULL;
    }
 
@@ -2146,7 +2149,7 @@ bson_new_from_buffer (uint8_t **buf,
       memcpy (*buf, &len_le, sizeof (len_le));
       (*buf)[4] = '\0';
    } else {
-      if ((*buf_len < 5) || (*buf_len > INT_MAX)) {
+      if ((*buf_len < 5) || (*buf_len > BSON_MAX_SIZE)) {
          bson_free (bson);
          return NULL;
       }
@@ -2246,11 +2249,11 @@ should_ignore (const char *first_exclude, va_list args, const char *name)
 }
 
 
-static void
-_bson_copy_to_excluding_va (const bson_t *src,
-                            bson_t *dst,
-                            const char *first_exclude,
-                            va_list args)
+void
+bson_copy_to_excluding_noinit_va (const bson_t *src,
+                                  bson_t *dst,
+                                  const char *first_exclude,
+                                  va_list args)
 {
    bson_iter_t iter;
 
@@ -2286,7 +2289,7 @@ bson_copy_to_excluding (const bson_t *src,
    bson_init (dst);
 
    va_start (args, first_exclude);
-   _bson_copy_to_excluding_va (src, dst, first_exclude, args);
+   bson_copy_to_excluding_noinit_va (src, dst, first_exclude, args);
    va_end (args);
 }
 
@@ -2303,7 +2306,7 @@ bson_copy_to_excluding_noinit (const bson_t *src,
    BSON_ASSERT (first_exclude);
 
    va_start (args, first_exclude);
-   _bson_copy_to_excluding_va (src, dst, first_exclude, args);
+   bson_copy_to_excluding_noinit_va (src, dst, first_exclude, args);
    va_end (args);
 }
 
@@ -2510,7 +2513,7 @@ bson_compare (const bson_t *bson, const bson_t *other)
    ret = memcmp (data1, data2, BSON_MIN (len1, len2));
 
    if (ret == 0) {
-      ret = (int64_t) (len1 - len2);
+      ret = (int64_t) len1 - (int64_t) len2;
    }
 
    return (ret < 0) ? -1 : (ret > 0);
@@ -2577,7 +2580,7 @@ _bson_as_json_visit_int64 (const bson_iter_t *iter,
 
    if (state->mode == BSON_JSON_MODE_CANONICAL) {
       bson_string_append_printf (
-         state->str, "{ \"$numberLong\" : \"%" PRId64 "\"}", v_int64);
+         state->str, "{ \"$numberLong\" : \"%" PRId64 "\" }", v_int64);
    } else {
       bson_string_append_printf (state->str, "%" PRId64, v_int64);
    }
@@ -2707,13 +2710,14 @@ _bson_as_json_visit_binary (const bson_iter_t *iter,
    size_t b64_len;
    char *b64;
 
-   b64_len = (v_binary_len / 3 + 1) * 4 + 1;
+   b64_len = COMMON_PREFIX (bson_b64_ntop_calculate_target_size (v_binary_len));
    b64 = bson_malloc0 (b64_len);
-   BSON_ASSERT (bson_b64_ntop (v_binary, v_binary_len, b64, b64_len) != -1);
+   BSON_ASSERT (
+      COMMON_PREFIX (bson_b64_ntop (v_binary, v_binary_len, b64, b64_len) != -1));
 
    if (state->mode == BSON_JSON_MODE_CANONICAL ||
        state->mode == BSON_JSON_MODE_RELAXED) {
-      bson_string_append (state->str, "{ \"$binary\" : { \"base64\": \"");
+      bson_string_append (state->str, "{ \"$binary\" : { \"base64\" : \"");
       bson_string_append (state->str, b64);
       bson_string_append (state->str, "\", \"subType\" : \"");
       bson_string_append_printf (state->str, "%02x", v_subtype);
@@ -2915,6 +2919,10 @@ _bson_as_json_visit_before (const bson_iter_t *iter,
    bson_json_state_t *state = data;
    char *escaped;
 
+   if (state->max_len_reached) {
+      return true;
+   }
+
    if (state->count) {
       bson_string_append (state->str, ", ");
    }
@@ -2937,6 +2945,30 @@ _bson_as_json_visit_before (const bson_iter_t *iter,
 }
 
 
+static bool
+_bson_as_json_visit_after (const bson_iter_t *iter, const char *key, void *data)
+{
+   bson_json_state_t *state = data;
+
+   if (state->max_len == BSON_MAX_LEN_UNLIMITED) {
+      return false;
+   }
+
+   if (state->str->len >= state->max_len) {
+      state->max_len_reached = true;
+
+      if (state->str->len > state->max_len) {
+         /* Truncate string to maximum length */
+         bson_string_truncate (state->str, state->max_len);
+      }
+
+      return true;
+   }
+
+   return false;
+}
+
+
 static void
 _bson_as_json_visit_corrupt (const bson_iter_t *iter, void *data)
 {
@@ -3011,27 +3043,33 @@ _bson_as_json_visit_codewscope (const bson_iter_t *iter,
    bson_json_state_t *state = data;
    char *code_escaped;
    char *scope;
+   int32_t max_scope_len = BSON_MAX_LEN_UNLIMITED;
 
    code_escaped = bson_utf8_escape_for_json (v_code, v_code_len);
    if (!code_escaped) {
       return true;
    }
 
-   /* Encode scope with the same mode */
-   scope = _bson_as_json_visit_all (v_scope, NULL, state->mode);
-
-   if (!scope) {
-      bson_free (code_escaped);
-      return true;
-   }
-
    bson_string_append (state->str, "{ \"$code\" : \"");
    bson_string_append (state->str, code_escaped);
    bson_string_append (state->str, "\", \"$scope\" : ");
+
+   bson_free (code_escaped);
+
+   /* Encode scope with the same mode */
+   if (state->max_len != BSON_MAX_LEN_UNLIMITED) {
+      max_scope_len = BSON_MAX (0, state->max_len - state->str->len);
+   }
+
+   scope = _bson_as_json_visit_all (v_scope, NULL, state->mode, max_scope_len);
+
+   if (!scope) {
+      return true;
+   }
+
    bson_string_append (state->str, scope);
    bson_string_append (state->str, " }");
 
-   bson_free (code_escaped);
    bson_free (scope);
 
    return false;
@@ -3039,7 +3077,7 @@ _bson_as_json_visit_codewscope (const bson_iter_t *iter,
 
 
 static const bson_visitor_t bson_as_json_visitors = {
-   _bson_as_json_visit_before,     NULL, /* visit_after */
+   _bson_as_json_visit_before,     _bson_as_json_visit_after,
    _bson_as_json_visit_corrupt,    _bson_as_json_visit_double,
    _bson_as_json_visit_utf8,       _bson_as_json_visit_document,
    _bson_as_json_visit_array,      _bson_as_json_visit_binary,
@@ -3074,9 +3112,24 @@ _bson_as_json_visit_document (const bson_iter_t *iter,
       child_state.str = bson_string_new ("{ ");
       child_state.depth = state->depth + 1;
       child_state.mode = state->mode;
+      child_state.max_len = BSON_MAX_LEN_UNLIMITED;
+      if (state->max_len != BSON_MAX_LEN_UNLIMITED) {
+         child_state.max_len = BSON_MAX (0, state->max_len - state->str->len);
+      }
+
+      child_state.max_len_reached = child_state.max_len == 0;
+
       if (bson_iter_visit_all (&child, &bson_as_json_visitors, &child_state)) {
+         if (child_state.max_len_reached) {
+            bson_string_append (state->str, child_state.str->str);
+         }
+
          bson_string_free (child_state.str, true);
-         return true;
+
+         /* If max_len was reached, we return a success state to ensure that
+          * VISIT_AFTER is still called
+          */
+         return !child_state.max_len_reached;
       }
 
       bson_string_append (child_state.str, " }");
@@ -3107,9 +3160,24 @@ _bson_as_json_visit_array (const bson_iter_t *iter,
       child_state.str = bson_string_new ("[ ");
       child_state.depth = state->depth + 1;
       child_state.mode = state->mode;
+      child_state.max_len = BSON_MAX_LEN_UNLIMITED;
+      if (state->max_len != BSON_MAX_LEN_UNLIMITED) {
+         child_state.max_len = BSON_MAX (0, state->max_len - state->str->len);
+      }
+
+      child_state.max_len_reached = child_state.max_len == 0;
+
       if (bson_iter_visit_all (&child, &bson_as_json_visitors, &child_state)) {
+         if (child_state.max_len_reached) {
+            bson_string_append (state->str, child_state.str->str);
+         }
+
          bson_string_free (child_state.str, true);
-         return true;
+
+         /* If max_len was reached, we return a success state to ensure that
+          * VISIT_AFTER is still called
+          */
+         return !child_state.max_len_reached;
       }
 
       bson_string_append (child_state.str, " ]");
@@ -3124,11 +3192,13 @@ _bson_as_json_visit_array (const bson_iter_t *iter,
 static char *
 _bson_as_json_visit_all (const bson_t *bson,
                          size_t *length,
-                         bson_json_mode_t mode)
+                         bson_json_mode_t mode,
+                         int32_t max_len)
 {
    bson_json_state_t state;
    bson_iter_t iter;
    ssize_t err_offset = -1;
+   int32_t remaining;
 
    BSON_ASSERT (bson);
 
@@ -3154,9 +3224,12 @@ _bson_as_json_visit_all (const bson_t *bson,
    state.depth = 0;
    state.err_offset = &err_offset;
    state.mode = mode;
+   state.max_len = max_len;
+   state.max_len_reached = false;
 
-   if (bson_iter_visit_all (&iter, &bson_as_json_visitors, &state) ||
-       err_offset != -1) {
+   if ((bson_iter_visit_all (&iter, &bson_as_json_visitors, &state) ||
+        err_offset != -1) &&
+       !state.max_len_reached) {
       /*
        * We were prematurely exited due to corruption or failed visitor.
        */
@@ -3167,7 +3240,14 @@ _bson_as_json_visit_all (const bson_t *bson,
       return NULL;
    }
 
-   bson_string_append (state.str, " }");
+   /* Append closing space and } separately, in case we hit the max in between. */
+   remaining = state.max_len - state.str->len;
+   if (state.max_len == BSON_MAX_LEN_UNLIMITED ||
+       remaining > 1) {
+      bson_string_append (state.str, " }");
+   } else if (remaining == 1) {
+      bson_string_append (state.str, " ");
+   }
 
    if (length) {
       *length = state.str->len;
@@ -3177,24 +3257,39 @@ _bson_as_json_visit_all (const bson_t *bson,
 }
 
 
+char *
+bson_as_json_with_opts (const bson_t *bson,
+                        size_t *length,
+                        const bson_json_opts_t *opts)
+{
+   return _bson_as_json_visit_all (bson, length, opts->mode, opts->max_len);
+}
+
+
 char *
 bson_as_canonical_extended_json (const bson_t *bson, size_t *length)
 {
-   return _bson_as_json_visit_all (bson, length, BSON_JSON_MODE_CANONICAL);
+   const bson_json_opts_t opts = {BSON_JSON_MODE_CANONICAL,
+                                  BSON_MAX_LEN_UNLIMITED};
+   return bson_as_json_with_opts (bson, length, &opts);
 }
 
 
 char *
 bson_as_json (const bson_t *bson, size_t *length)
 {
-   return _bson_as_json_visit_all (bson, length, BSON_JSON_MODE_LEGACY);
+   const bson_json_opts_t opts = {BSON_JSON_MODE_LEGACY,
+                                  BSON_MAX_LEN_UNLIMITED};
+   return bson_as_json_with_opts (bson, length, &opts);
 }
 
 
 char *
 bson_as_relaxed_extended_json (const bson_t *bson, size_t *length)
 {
-   return _bson_as_json_visit_all (bson, length, BSON_JSON_MODE_RELAXED);
+   const bson_json_opts_t opts = {BSON_JSON_MODE_RELAXED,
+                                  BSON_MAX_LEN_UNLIMITED};
+   return bson_as_json_with_opts (bson, length, &opts);
 }
 
 
@@ -3209,7 +3304,7 @@ bson_array_as_json (const bson_t *bson, size_t *length)
 
    if (length) {
       *length = 0;
-   }
+    }
 
    if (bson_empty0 (bson)) {
       if (length) {
@@ -3229,9 +3324,12 @@ bson_array_as_json (const bson_t *bson, size_t *length)
    state.depth = 0;
    state.err_offset = &err_offset;
    state.mode = BSON_JSON_MODE_LEGACY;
+   state.max_len = BSON_MAX_LEN_UNLIMITED;
+   state.max_len_reached = false;
 
-   if (bson_iter_visit_all (&iter, &bson_as_json_visitors, &state) ||
-       err_offset != -1) {
+   if ((bson_iter_visit_all (&iter, &bson_as_json_visitors, &state) ||
+        err_offset != -1) &&
+       !state.max_len_reached) {
       /*
        * We were prematurely exited due to corruption or failed visitor.
        */
