@@ -19,6 +19,14 @@ skipped_corpus_test_t SKIPPED_CORPUS_TESTS[] = {
     "Unicode and embedded null in code string, empty scope"},
    /* CDRIVER-2223, legacy extended JSON $date syntax uses numbers */
    {"Top-level document validity", "Bad $date (number, not string or hash)"},
+   /* CDRIVER-3500, floating point output differs */
+   {"Double type", "1.2345678921232E+18"},
+   {"Double type", "-1.2345678921232E+18"},
+   /* CDRIVER-4017, libbson does not emit escape sequences */
+   {"Javascript Code", "two-byte UTF-8 (\xc3\xa9)"}, /* \u00e9 */
+   {"Javascript Code", "three-byte UTF-8 (\xe2\x98\x86)"}, /* \u2606 */
+   {"String", "two-byte UTF-8 (\xc3\xa9)"}, /* \u00e9 */
+   {"String", "three-byte UTF-8 (\xe2\x98\x86)"}, /* \u2606 */
    {0}};
 
 
@@ -239,6 +247,7 @@ test_bson_corpus_parse_error (test_bson_parse_error_type_t *test)
 
    switch (test->bson_type) {
    case BSON_TYPE_EOD: /* top-level document to be parsed as JSON */
+   case BSON_TYPE_BINARY:
       ASSERT (!bson_new_from_json ((uint8_t *) test->str, test->str_len, NULL));
       break;
    case BSON_TYPE_DECIMAL128: {
@@ -251,7 +260,6 @@ test_bson_corpus_parse_error (test_bson_parse_error_type_t *test)
    case BSON_TYPE_UTF8:
    case BSON_TYPE_DOCUMENT:
    case BSON_TYPE_ARRAY:
-   case BSON_TYPE_BINARY:
    case BSON_TYPE_UNDEFINED:
    case BSON_TYPE_OID:
    case BSON_TYPE_BOOL:
