@@ -14,17 +14,19 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_TOPOLOGY_PRIVATE_H
 #define MONGOC_TOPOLOGY_PRIVATE_H
 
-#include "mongoc/mongoc-topology-scanner-private.h"
-#include "mongoc/mongoc-server-description-private.h"
-#include "mongoc/mongoc-topology-description-private.h"
-#include "mongoc/mongoc-thread-private.h"
-#include "mongoc/mongoc-uri.h"
-#include "mongoc/mongoc-client-session-private.h"
+#include "mongoc-config.h"
+#include "mongoc-topology-scanner-private.h"
+#include "mongoc-server-description-private.h"
+#include "mongoc-topology-description-private.h"
+#include "mongoc-thread-private.h"
+#include "mongoc-uri.h"
+#include "mongoc-client-session-private.h"
+#include "mongoc-crypt-private.h"
 
 #define MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS 500
 #define MONGOC_TOPOLOGY_SOCKET_CHECK_INTERVAL_MS 5000
@@ -38,12 +40,20 @@
 typedef enum {
    MONGOC_TOPOLOGY_SCANNER_OFF,
    MONGOC_TOPOLOGY_SCANNER_BG_RUNNING,
-   MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN,
-   MONGOC_TOPOLOGY_SCANNER_SINGLE_THREADED,
+   MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN
 } mongoc_topology_scanner_state_t;
 
+struct _mongoc_background_monitor_t;
+struct _mongoc_client_pool_t;
+
 typedef struct _mongoc_topology_t {
    mongoc_topology_description_t description;
+   /* topology->uri is initialized as a copy of the client/pool's URI.
+    * For a "mongodb+srv://" URI, topology->uri is then updated in
+    * mongoc_topology_new() after initial seedlist discovery.
+    * Afterwards, it remains read-only and may be read outside of the topology
+    * mutex.
+    */
    mongoc_uri_t *uri;
    mongoc_topology_scanner_t *scanner;
    bool server_selection_try_once;
@@ -57,20 +67,43 @@ typedef struct _mongoc_topology_t {
 
    /* Minimum of SRV record TTLs, but no lower than 60 seconds.
     * May be zero for non-SRV/non-MongoS topology. */
-   int64_t rescanSRVIntervalMS;
-   int64_t last_srv_scan;
+   int64_t srv_polling_rescan_interval_ms;
+   int64_t srv_polling_last_scan_ms;
+   /* For multi-threaded, srv polling occurs in a separate thread. */
+   bson_thread_t srv_polling_thread;
+   mongoc_cond_t srv_polling_cond;
 
    bson_mutex_t mutex;
    mongoc_cond_t cond_client;
-   mongoc_cond_t cond_server;
-   bson_thread_t thread;
-
    mongoc_topology_scanner_state_t scanner_state;
-   bool scan_requested;
+
    bool single_threaded;
    bool stale;
 
    mongoc_server_session_t *session_pool;
+
+   /* Is client side encryption enabled? */
+   bool cse_enabled;
+   bool is_srv_polling;
+
+#ifdef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
+   _mongoc_crypt_t *crypt;
+   struct _mongoc_client_t *mongocryptd_client;           /* single threaded */
+   struct _mongoc_client_t *keyvault_client;              /* single threaded */
+   struct _mongoc_client_pool_t *mongocryptd_client_pool; /* multi threaded */
+   struct _mongoc_client_pool_t *keyvault_client_pool;    /* multi threaded */
+   char *keyvault_db;
+   char *keyvault_coll;
+   bool bypass_auto_encryption;
+   bool mongocryptd_bypass_spawn;
+   char *mongocryptd_spawn_path;
+   bson_t *mongocryptd_spawn_args;
+#endif
+
+   /* For background monitoring. */
+   mongoc_set_t *server_monitors;
+   mongoc_set_t *rtt_monitors;
+   bson_mutex_t apm_mutex;
 } mongoc_topology_t;
 
 mongoc_topology_t *
@@ -114,6 +147,12 @@ _mongoc_topology_host_by_id (mongoc_topology_t *topology,
                              uint32_t id,
                              bson_error_t *error);
 
+/* TODO: Try to remove this function when CDRIVER-3654 is complete.
+ * It is only called when an application thread needs to mark a server Unknown.
+ * But an application error is also tied to other behavior, and should also
+ * consider the connection generation. This logic is captured in
+ * _mongoc_topology_handle_app_error. This should not be called directly
+ */
 void
 mongoc_topology_invalidate_server (mongoc_topology_t *topology,
                                    uint32_t id,
@@ -133,12 +172,6 @@ mongoc_topology_server_timestamp (mongoc_topology_t *topology, uint32_t id);
 mongoc_topology_description_type_t
 _mongoc_topology_get_type (mongoc_topology_t *topology);
 
-bool
-_mongoc_topology_start_background_scanner (mongoc_topology_t *topology);
-
-void
-_mongoc_topology_background_thread_stop (mongoc_topology_t *topology);
-
 bool
 _mongoc_topology_set_appname (mongoc_topology_t *topology, const char *appname);
 
@@ -164,7 +197,36 @@ void
 _mongoc_topology_do_blocking_scan (mongoc_topology_t *topology,
                                    bson_error_t *error);
 const bson_t *
-_mongoc_topology_get_ismaster (mongoc_topology_t *topology);
+_mongoc_topology_get_handshake_cmd (mongoc_topology_t *topology);
 void
 _mongoc_topology_request_scan (mongoc_topology_t *topology);
+
+void
+_mongoc_topology_bypass_cooldown (mongoc_topology_t *topology);
+
+typedef enum {
+   MONGOC_SDAM_APP_ERROR_COMMAND,
+   MONGOC_SDAM_APP_ERROR_NETWORK,
+   MONGOC_SDAM_APP_ERROR_TIMEOUT
+} _mongoc_sdam_app_error_type_t;
+
+bool
+_mongoc_topology_handle_app_error (mongoc_topology_t *topology,
+                                   uint32_t server_id,
+                                   bool handshake_complete,
+                                   _mongoc_sdam_app_error_type_t type,
+                                   const bson_t *reply,
+                                   const bson_error_t *why,
+                                   uint32_t max_wire_version,
+                                   uint32_t generation);
+
+void
+_mongoc_topology_clear_connection_pool (mongoc_topology_t *topology,
+                                        uint32_t server_id);
+
+void
+mongoc_topology_rescan_srv (mongoc_topology_t *topology);
+
+bool
+mongoc_topology_should_rescan_srv (mongoc_topology_t *topology);
 #endif
