@@ -51,7 +51,7 @@ _async_connected (mongoc_async_cmd_t *acmd);
 
 static void
 _async_success (mongoc_async_cmd_t *acmd,
-                const bson_t *ismaster_response,
+                const bson_t *hello_response,
                 int64_t duration_usec);
 
 static void
@@ -62,7 +62,7 @@ _async_error_or_timeout (mongoc_async_cmd_t *acmd,
 static void
 _async_handler (mongoc_async_cmd_t *acmd,
                 mongoc_async_cmd_result_t async_status,
-                const bson_t *ismaster_response,
+                const bson_t *hello_response,
                 int64_t duration_usec);
 
 static void
@@ -104,9 +104,40 @@ _jumpstart_other_acmds (mongoc_topology_scanner_node_t *node,
                         mongoc_async_cmd_t *acmd);
 
 static void
-_add_ismaster (bson_t *cmd)
+_add_hello (mongoc_topology_scanner_t *ts)
 {
-   BSON_APPEND_INT32 (cmd, "isMaster", 1);
+   mongoc_server_api_t *api = ts->api;
+
+   BSON_APPEND_INT32 (&ts->hello_cmd, "hello", 1);
+   BSON_APPEND_BOOL (&ts->hello_cmd, "helloOk", true);
+
+   BSON_APPEND_INT32 (&ts->legacy_hello_cmd, HANDSHAKE_CMD_LEGACY_HELLO, 1);
+   BSON_APPEND_BOOL (&ts->legacy_hello_cmd, "helloOk", true);
+
+   if (api) {
+      _mongoc_cmd_append_server_api (&ts->hello_cmd, api);
+   }
+}
+
+static void
+_init_hello (mongoc_topology_scanner_t *ts)
+{
+   bson_init (&ts->hello_cmd);
+   bson_init (&ts->legacy_hello_cmd);
+   bson_init (&ts->handshake_cmd);
+   bson_init (&ts->cluster_time);
+
+   _add_hello (ts);
+}
+
+static void
+_reset_hello (mongoc_topology_scanner_t *ts)
+{
+   bson_reinit (&ts->hello_cmd);
+   bson_reinit (&ts->legacy_hello_cmd);
+   bson_reinit (&ts->handshake_cmd);
+
+   _add_hello (ts);
 }
 
 const char *
@@ -192,17 +223,17 @@ _mongoc_topology_scanner_add_speculative_authentication (
 
 void
 _mongoc_topology_scanner_parse_speculative_authentication (
-   const bson_t *ismaster, bson_t *speculative_authenticate)
+   const bson_t *hello, bson_t *speculative_authenticate)
 {
    bson_iter_t iter;
    uint32_t data_len;
    const uint8_t *data;
    bson_t auth_response;
 
-   BSON_ASSERT (ismaster);
+   BSON_ASSERT (hello);
    BSON_ASSERT (speculative_authenticate);
 
-   if (!bson_iter_init_find (&iter, ismaster, "speculativeAuthenticate")) {
+   if (!bson_iter_init_find (&iter, hello, "speculativeAuthenticate")) {
       return;
    }
 
@@ -214,9 +245,9 @@ _mongoc_topology_scanner_parse_speculative_authentication (
 }
 
 static bool
-_build_ismaster_with_handshake (mongoc_topology_scanner_t *ts)
+_build_handshake_cmd (mongoc_topology_scanner_t *ts)
 {
-   bson_t *doc = &ts->ismaster_cmd_with_handshake;
+   bson_t *doc = &ts->handshake_cmd;
    bson_t subdoc;
    bson_iter_t iter;
    const char *key;
@@ -226,7 +257,8 @@ _build_ismaster_with_handshake (mongoc_topology_scanner_t *ts)
    int count = 0;
    char buf[16];
 
-   _add_ismaster (doc);
+   bson_destroy (doc);
+   bson_copy_to (ts->api ? &ts->hello_cmd : &ts->legacy_hello_cmd, doc);
 
    BSON_APPEND_DOCUMENT_BEGIN (doc, HANDSHAKE_FIELD, &subdoc);
    res = _mongoc_handshake_build_doc_with_application (&subdoc, ts->appname);
@@ -250,45 +282,53 @@ _build_ismaster_with_handshake (mongoc_topology_scanner_t *ts)
    return res;
 }
 
-/* Caller must lock topology->mutex to protect ismaster_cmd_with_handshake. This
- * is called at the start of the scan in _mongoc_topology_run_background, when a
- * node is added in _mongoc_topology_reconcile_add_nodes, or when running an
- * ismaster directly on a node in _mongoc_stream_run_ismaster. */
 const bson_t *
-_mongoc_topology_scanner_get_ismaster (mongoc_topology_scanner_t *ts)
+_mongoc_topology_scanner_get_monitoring_cmd (mongoc_topology_scanner_t *ts,
+                                             bool hello_ok)
+{
+   return hello_ok || ts->api ? &ts->hello_cmd : &ts->legacy_hello_cmd;
+}
+
+/* Caller must lock topology->mutex to protect handshake_cmd. This
+ * is called at the start of the scan in _mongoc_topology_run_background, when a
+ * node is added in _mongoc_topology_reconcile_add_nodes, or when running a
+ * hello directly on a node in _mongoc_stream_run_hello. */
+const bson_t *
+_mongoc_topology_scanner_get_handshake_cmd (mongoc_topology_scanner_t *ts)
 {
    /* If this is the first time using the node or if it's the first time
     * using it after a failure, build handshake doc */
-   if (bson_empty (&ts->ismaster_cmd_with_handshake)) {
-      ts->handshake_ok_to_send = _build_ismaster_with_handshake (ts);
+   if (bson_empty (&ts->handshake_cmd)) {
+      ts->handshake_ok_to_send = _build_handshake_cmd (ts);
       if (!ts->handshake_ok_to_send) {
-         MONGOC_WARNING ("Handshake doc too big, not including in isMaster");
+         MONGOC_WARNING ("Handshake doc too big, not including in hello");
       }
    }
 
    /* If the doc turned out to be too big */
    if (!ts->handshake_ok_to_send) {
-      return &ts->ismaster_cmd;
+      return ts->api ? &ts->hello_cmd : &ts->legacy_hello_cmd;
    }
 
-   return &ts->ismaster_cmd_with_handshake;
+   return &ts->handshake_cmd;
 }
 
 static void
-_begin_ismaster_cmd (mongoc_topology_scanner_node_t *node,
-                     mongoc_stream_t *stream,
-                     bool is_setup_done,
-                     struct addrinfo *dns_result,
-                     int64_t initiate_delay_ms)
+_begin_hello_cmd (mongoc_topology_scanner_node_t *node,
+                  mongoc_stream_t *stream,
+                  bool is_setup_done,
+                  struct addrinfo *dns_result,
+                  int64_t initiate_delay_ms)
 {
    mongoc_topology_scanner_t *ts = node->ts;
    bson_t cmd;
 
    if (node->last_used != -1 && node->last_failed == -1) {
       /* The node's been used before and not failed recently */
-      bson_copy_to (&ts->ismaster_cmd, &cmd);
+      bson_copy_to (_mongoc_topology_scanner_get_monitoring_cmd (ts, node->hello_ok),
+                    &cmd);
    } else {
-      bson_copy_to (_mongoc_topology_scanner_get_ismaster (ts), &cmd);
+      bson_copy_to (_mongoc_topology_scanner_get_handshake_cmd (ts), &cmd);
    }
 
    if (node->ts->negotiate_sasl_supported_mechs &&
@@ -346,21 +386,19 @@ mongoc_topology_scanner_new (
 
    ts->async = mongoc_async_new ();
 
-   bson_init (&ts->ismaster_cmd);
-   _add_ismaster (&ts->ismaster_cmd);
-   bson_init (&ts->ismaster_cmd_with_handshake);
-   bson_init (&ts->cluster_time);
-
    ts->setup_err_cb = setup_err_cb;
    ts->cb = cb;
    ts->cb_data = data;
    ts->uri = uri;
    ts->appname = NULL;
+   ts->api = NULL;
    ts->handshake_ok_to_send = false;
    ts->connect_timeout_msec = connect_timeout_msec;
    /* may be overridden for testing. */
    ts->dns_cache_timeout_ms = DNS_CACHE_TIMEOUT_MS;
 
+   _init_hello (ts);
+
    return ts;
 }
 
@@ -395,9 +433,11 @@ mongoc_topology_scanner_destroy (mongoc_topology_scanner_t *ts)
    }
 
    mongoc_async_destroy (ts->async);
-   bson_destroy (&ts->ismaster_cmd);
-   bson_destroy (&ts->ismaster_cmd_with_handshake);
+   bson_destroy (&ts->hello_cmd);
+   bson_destroy (&ts->legacy_hello_cmd);
+   bson_destroy (&ts->handshake_cmd);
    bson_destroy (&ts->cluster_time);
+   mongoc_server_api_destroy (ts->api);
 
    /* This field can be set by a mongoc_client */
    bson_free ((char *) ts->appname);
@@ -416,7 +456,8 @@ mongoc_topology_scanner_valid (mongoc_topology_scanner_t *ts)
 void
 mongoc_topology_scanner_add (mongoc_topology_scanner_t *ts,
                              const mongoc_host_list_t *host,
-                             uint32_t id)
+                             uint32_t id,
+                             bool hello_ok)
 {
    mongoc_topology_scanner_node_t *node;
 
@@ -428,6 +469,7 @@ mongoc_topology_scanner_add (mongoc_topology_scanner_t *ts,
    node->ts = ts;
    node->last_failed = -1;
    node->last_used = -1;
+   node->hello_ok = hello_ok;
    bson_init (&node->speculative_auth_response);
 
    DL_APPEND (ts->nodes, node);
@@ -575,7 +617,7 @@ _async_connected (mongoc_async_cmd_t *acmd)
 
 static void
 _async_success (mongoc_async_cmd_t *acmd,
-                const bson_t *ismaster_response,
+                const bson_t *hello_response,
                 int64_t duration_usec)
 {
    void *data = acmd->data;
@@ -595,7 +637,7 @@ _async_success (mongoc_async_cmd_t *acmd,
    node->last_failed = -1;
 
    _mongoc_topology_scanner_monitor_heartbeat_succeeded (
-      ts, &node->host, ismaster_response, duration_usec);
+      ts, &node->host, hello_response, duration_usec);
 
    /* set our successful stream. */
    BSON_ASSERT (!node->stream);
@@ -604,17 +646,17 @@ _async_success (mongoc_async_cmd_t *acmd,
    if (ts->negotiate_sasl_supported_mechs &&
        !node->negotiated_sasl_supported_mechs) {
       _mongoc_handshake_parse_sasl_supported_mechs (
-         ismaster_response, &node->sasl_supported_mechs);
+         hello_response, &node->sasl_supported_mechs);
    }
 
    if (ts->speculative_authentication) {
       _mongoc_topology_scanner_parse_speculative_authentication (
-         ismaster_response, &node->speculative_auth_response);
+         hello_response, &node->speculative_auth_response);
    }
 
    /* mongoc_topology_scanner_cb_t takes rtt_msec, not usec */
    ts->cb (node->id,
-           ismaster_response,
+           hello_response,
            duration_usec / 1000,
            ts->cb_data,
            &acmd->error);
@@ -664,7 +706,7 @@ _async_error_or_timeout (mongoc_async_cmd_t *acmd,
       bson_set_error (&node->last_error,
                       MONGOC_ERROR_CLIENT,
                       MONGOC_ERROR_STREAM_CONNECT,
-                      "%s calling ismaster on \'%s\'",
+                      "%s calling hello on \'%s\'",
                       message,
                       node->host.host_and_port);
 
@@ -685,7 +727,7 @@ _async_error_or_timeout (mongoc_async_cmd_t *acmd,
  *-----------------------------------------------------------------------
  *
  * This is the callback passed to async_cmd when we're running
- * ismasters from within the topology monitor.
+ * hellos from within the topology monitor.
  *
  *-----------------------------------------------------------------------
  */
@@ -693,7 +735,7 @@ _async_error_or_timeout (mongoc_async_cmd_t *acmd,
 static void
 _async_handler (mongoc_async_cmd_t *acmd,
                 mongoc_async_cmd_result_t async_status,
-                const bson_t *ismaster_response,
+                const bson_t *hello_response,
                 int64_t duration_usec)
 {
    BSON_ASSERT (acmd->data);
@@ -703,7 +745,7 @@ _async_handler (mongoc_async_cmd_t *acmd,
       _async_connected (acmd);
       return;
    case MONGOC_ASYNC_CMD_SUCCESS:
-      _async_success (acmd, ismaster_response, duration_usec);
+      _async_success (acmd, hello_response, duration_usec);
       return;
    case MONGOC_ASYNC_CMD_TIMEOUT:
       _async_error_or_timeout (acmd, duration_usec, "connection timeout");
@@ -829,11 +871,11 @@ mongoc_topology_scanner_node_setup_tcp (mongoc_topology_scanner_node_t *node,
    }
 
    if (node->successful_dns_result) {
-      _begin_ismaster_cmd (node, NULL, false, node->successful_dns_result, 0);
+      _begin_hello_cmd (node, NULL, false, node->successful_dns_result, 0);
    } else {
       LL_FOREACH2 (node->dns_results, iter, ai_next)
       {
-         _begin_ismaster_cmd (node, NULL, false, iter, delay);
+         _begin_hello_cmd (node, NULL, false, iter, delay);
          /* each subsequent DNS result will have an additional 250ms delay. */
          delay += HAPPY_EYEBALLS_DELAY_MS;
       }
@@ -896,11 +938,11 @@ mongoc_topology_scanner_node_connect_unix (mongoc_topology_scanner_node_t *node,
    stream = _mongoc_topology_scanner_node_setup_stream_for_tls (
       node, mongoc_stream_socket_new (sock));
    if (stream) {
-      _begin_ismaster_cmd (node,
-                           stream,
-                           false /* is_setup_done */,
-                           NULL /* dns result */,
-                           0 /* delay */);
+      _begin_hello_cmd (node,
+                        stream,
+                        false /* is_setup_done */,
+                        NULL /* dns result */,
+                        0 /* delay */);
       RETURN (true);
    }
    bson_set_error (error,
@@ -938,8 +980,7 @@ mongoc_topology_scanner_node_setup (mongoc_topology_scanner_node_t *node,
 
    /* if there is already a working stream, push it back to be re-scanned. */
    if (node->stream) {
-      _begin_ismaster_cmd (
-         node, node->stream, true /* is_setup_done */, NULL, 0);
+      _begin_hello_cmd (node, node->stream, true /* is_setup_done */, NULL, 0);
       node->stream = NULL;
       return;
    }
@@ -951,7 +992,7 @@ mongoc_topology_scanner_node_setup (mongoc_topology_scanner_node_t *node,
          node->ts->uri, &node->host, node->ts->initiator_context, error);
       if (stream) {
          success = true;
-         _begin_ismaster_cmd (node, stream, false, NULL, 0);
+         _begin_hello_cmd (node, stream, false, NULL, 0);
       }
    } else {
       if (node->host.family == AF_UNIX) {
@@ -981,7 +1022,7 @@ mongoc_topology_scanner_node_setup (mongoc_topology_scanner_node_t *node,
  * mongoc_topology_scanner_node_in_cooldown --
  *
  *      Return true if @node has experienced a network error attempting
- *      to call "ismaster" less than 5 seconds before @when, a timestamp in
+ *      to call "hello" less than 5 seconds before @when, a timestamp in
  *      microseconds.
  *
  *      Server Discovery and Monitoring Spec: "After a single-threaded client
@@ -1224,11 +1265,11 @@ _mongoc_topology_scanner_monitor_heartbeat_succeeded (
 {
    if (ts->apm_callbacks.server_heartbeat_succeeded) {
       mongoc_apm_server_heartbeat_succeeded_t event;
-      bson_t ismaster_redacted;
+      bson_t hello_redacted;
 
-      bson_init (&ismaster_redacted);
+      bson_init (&hello_redacted);
       bson_copy_to_excluding_noinit (
-         reply, &ismaster_redacted, "speculativeAuthenticate", NULL);
+         reply, &hello_redacted, "speculativeAuthenticate", NULL);
 
       event.host = host;
       event.context = ts->apm_context;
@@ -1237,7 +1278,7 @@ _mongoc_topology_scanner_monitor_heartbeat_succeeded (
       event.awaited = false;
       ts->apm_callbacks.server_heartbeat_succeeded (&event);
 
-      bson_destroy (&ismaster_redacted);
+      bson_destroy (&hello_redacted);
    }
 }
 
@@ -1324,3 +1365,16 @@ _jumpstart_other_acmds (mongoc_topology_scanner_node_t *node,
       }
    }
 }
+
+/* Caller must lock topology->mutex to protect handshake_cmd. */
+void
+_mongoc_topology_scanner_set_server_api (mongoc_topology_scanner_t *ts,
+                                         const mongoc_server_api_t *api)
+{
+   BSON_ASSERT (ts);
+   BSON_ASSERT (api);
+
+   mongoc_server_api_destroy (ts->api);
+   ts->api = mongoc_server_api_copy (api);
+   _reset_hello (ts);
+}
