@@ -2255,6 +2255,53 @@ assert_event_count (json_test_ctx_t *ctx, const bson_t *operation)
    }
 }
 
+static void
+assert_session_transaction_state (json_test_ctx_t *ctx, const bson_t *operation)
+{
+   const char *expected_state;
+   const mongoc_client_session_t *session;
+   mongoc_transaction_state_t state;
+   char *state_strings[] = {
+      "none", "starting", "in progress", "committed", "aborted"};
+
+   expected_state = bson_lookup_utf8 (operation, "arguments.state");
+   session = session_from_name (
+      ctx, bson_lookup_utf8 (operation, "arguments.session"));
+   state = mongoc_client_session_get_transaction_state (session);
+
+   if (!strcmp (expected_state, "none")) {
+      if (state != MONGOC_TRANSACTION_NONE) {
+         test_error ("expected session transaction state none, but have %s",
+                     state_strings[state]);
+      }
+   } else if (!strcmp (expected_state, "starting")) {
+      if (state != MONGOC_TRANSACTION_STARTING) {
+         test_error ("expected session transaction state starting, but have %s",
+                     state_strings[state]);
+      }
+   } else if (!strcmp (expected_state, "in_progress")) {
+      if (state != MONGOC_TRANSACTION_IN_PROGRESS) {
+         test_error (
+            "expected session transaction state in progress, but have %s",
+            state_strings[state]);
+      }
+   } else if (!strcmp (expected_state, "committed")) {
+      if (state != MONGOC_TRANSACTION_COMMITTED) {
+         test_error (
+            "expected session transaction state committed, but have %s",
+            state_strings[state]);
+      }
+   } else if (!strcmp (expected_state, "aborted")) {
+      if (state != MONGOC_TRANSACTION_ABORTED) {
+         test_error ("expected session transaction state aborted, but have %s",
+                     state_strings[state]);
+      }
+   } else {
+      test_error ("unrecognized state %s for assertSessionTransactionState",
+                  expected_state);
+   }
+}
+
 static bool
 op_error (const bson_t *operation)
 {
@@ -2429,7 +2476,7 @@ json_test_operation (json_test_ctx_t *ctx,
       } else if (!strcmp (op_name, "targetedFailPoint")) {
          mongoc_client_t *client;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          activate_fail_point (
             client, session->server_id, operation, "arguments.failPoint");
@@ -2455,7 +2502,7 @@ json_test_operation (json_test_ctx_t *ctx,
          mongoc_client_t *client;
          bool exists;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          exists = collection_exists (client, operation);
          mongoc_client_destroy (client);
@@ -2465,7 +2512,7 @@ json_test_operation (json_test_ctx_t *ctx,
          mongoc_client_t *client;
          bool exists;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          exists = collection_exists (client, operation);
          mongoc_client_destroy (client);
@@ -2475,7 +2522,7 @@ json_test_operation (json_test_ctx_t *ctx,
          mongoc_client_t *client;
          bool exists;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          exists = index_exists (client, operation);
          mongoc_client_destroy (client);
@@ -2485,7 +2532,7 @@ json_test_operation (json_test_ctx_t *ctx,
          mongoc_client_t *client;
          bool exists;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          exists = index_exists (client, operation);
          mongoc_client_destroy (client);
@@ -2498,7 +2545,7 @@ json_test_operation (json_test_ctx_t *ctx,
       } else if (!strcmp (op_name, "configureFailPoint")) {
          mongoc_client_t *client;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          activate_fail_point (
             client, 0 /* primary */, operation, "arguments.failPoint");
@@ -2517,7 +2564,7 @@ json_test_operation (json_test_ctx_t *ctx,
          mongoc_client_t *client;
          mongoc_database_t *admin_db;
 
-         client = mongoc_client_new_from_uri (ctx->client->uri);
+         client = test_framework_client_new_from_uri (ctx->client->uri, NULL);
          test_framework_set_ssl_opts (client);
          admin_db = mongoc_client_get_database (client, "admin");
          bson_destroy (reply);
@@ -2549,6 +2596,8 @@ json_test_operation (json_test_ctx_t *ctx,
          wt = thread_from_name (ctx,
                                 bson_lookup_utf8 (operation, "arguments.name"));
          run_on_thread (wt, &op);
+      } else if (!strcmp (op_name, "assertSessionTransactionState")) {
+         assert_session_transaction_state (ctx, operation);
       } else {
          test_error ("unrecognized testRunner operation name %s", op_name);
       }
