@@ -154,9 +154,9 @@ server_description_by_hostname (mongoc_topology_description_t *topology,
 /*
  *-----------------------------------------------------------------------
  *
- * process_sdam_test_ismaster_responses --
+ * process_sdam_test_hello_responses --
  *
- *      Update a topology description with the ismaster responses in a "phase"
+ *      Update a topology description with the hello responses in a "phase"
  *      from an SDAM or SDAM Monitoring test, like:
  *
  *      [
@@ -164,7 +164,7 @@ server_description_by_hostname (mongoc_topology_description_t *topology,
  *              "a:27017",
  *              {
  *                  "ok": 1,
- *                  "ismaster": false
+ *                  "isWritablePrimary": false
  *              }
  *          ]
  *      ]
@@ -175,8 +175,8 @@ server_description_by_hostname (mongoc_topology_description_t *topology,
  *-----------------------------------------------------------------------
  */
 void
-process_sdam_test_ismaster_responses (bson_t *phase,
-                                      mongoc_topology_t *topology)
+process_sdam_test_hello_responses (bson_t *phase,
+                                   mongoc_topology_t *topology)
 {
    mongoc_topology_description_t *td;
    mongoc_server_description_t *sd;
@@ -190,22 +190,22 @@ process_sdam_test_ismaster_responses (bson_t *phase,
       description = bson_iter_utf8 (&phase_field_iter, NULL);
       MONGOC_DEBUG ("phase: %s", description);
    }
-   /* grab ismaster responses out and feed them to topology */
+   /* grab hello responses out and feed them to topology */
    if (bson_iter_init_find (&phase_field_iter, phase, "responses")) {
-      bson_t ismasters;
-      bson_t ismaster;
+      bson_t hellos;
+      bson_t hello;
       bson_t response;
-      bson_iter_t ismaster_iter;
-      bson_iter_t ismaster_field_iter;
+      bson_iter_t hello_iter;
+      bson_iter_t hello_field_iter;
 
-      bson_iter_bson (&phase_field_iter, &ismasters);
-      bson_iter_init (&ismaster_iter, &ismasters);
+      bson_iter_bson (&phase_field_iter, &hellos);
+      bson_iter_init (&hello_iter, &hellos);
 
-      while (bson_iter_next (&ismaster_iter)) {
-         bson_iter_bson (&ismaster_iter, &ismaster);
+      while (bson_iter_next (&hello_iter)) {
+         bson_iter_bson (&hello_iter, &hello);
 
-         bson_iter_init_find (&ismaster_field_iter, &ismaster, "0");
-         hostname = bson_iter_utf8 (&ismaster_field_iter, NULL);
+         bson_iter_init_find (&hello_field_iter, &hello, "0");
+         hostname = bson_iter_utf8 (&hello_field_iter, NULL);
          sd = server_description_by_hostname (td, hostname);
 
          /* if server has been removed from topology, skip */
@@ -213,12 +213,12 @@ process_sdam_test_ismaster_responses (bson_t *phase,
             continue;
          }
 
-         bson_iter_init_find (&ismaster_field_iter, &ismaster, "1");
-         bson_iter_bson (&ismaster_field_iter, &response);
+         bson_iter_init_find (&hello_field_iter, &hello, "1");
+         bson_iter_bson (&hello_field_iter, &response);
 
-         /* send ismaster through the topology description's handler */
+         /* send hello through the topology description's handler */
          capture_logs (true);
-         mongoc_topology_description_handle_ismaster (
+         mongoc_topology_description_handle_hello (
             td, sd->id, &response, 1, NULL);
          if (td->servers->items_len == 0) {
             ASSERT_CAPTURED_LOG ("topology",
@@ -1051,7 +1051,7 @@ insert_data (const char *db_name,
    bson_t *majority = tmp_bson ("{'writeConcern': {'w': 'majority'}}");
 
    /* use a fresh client to prepare the collection */
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
 
    db = mongoc_client_get_database (client, db_name);
    collection = mongoc_database_get_collection (db, collection_name);
@@ -1194,17 +1194,23 @@ execute_test (const json_test_config_t *config,
 
    if (bson_has_field (test, "expectations")) {
       bson_t expectations;
+      bson_iter_t iter;
 
-      bson_lookup_doc (test, "expectations", &expectations);
-      check_json_apm_events (&ctx, &expectations);
-      if (config->events_check_cb) {
-         config->events_check_cb (&ctx.events);
+      if (bson_iter_init_find (&iter, test, "expectations")) {
+         /* If 'expectations' is explicitly null, skip the check. */
+         if (!BSON_ITER_HOLDS_NULL (&iter)) {
+            bson_iter_bson (&iter, &expectations);
+            check_json_apm_events (&ctx, &expectations);
+            if (config->events_check_cb) {
+               config->events_check_cb (&ctx.events);
+            }
+         }
       }
    }
 
    if (bson_has_field (test, "outcome.collection")) {
       /* Use a fresh client to check the outcome collection. */
-      outcome_client = test_framework_client_new ();
+      outcome_client = test_framework_new_default_client ();
       if (bson_has_field (test, "outcome.collection.name")) {
          outcome_coll = mongoc_client_get_collection (
             outcome_client,
@@ -1408,7 +1414,8 @@ set_auto_encryption_opts (mongoc_client_t *client, bson_t *test)
       bson_t tmp;
 
       bson_iter_bson (&iter, &tmp);
-      bson_copy_to_excluding (&tmp, &kms_providers, "aws", NULL);
+      bson_copy_to_excluding (
+         &tmp, &kms_providers, "aws", "azure", "gcp", NULL);
 
       /* AWS credentials are set from environment variables. */
       if (bson_has_field (&opts, "kmsProviders.aws")) {
@@ -1439,6 +1446,64 @@ set_auto_encryption_opts (mongoc_client_t *client, bson_t *test)
          bson_free (aws_access_key_id);
       }
 
+      if (bson_has_field (&opts, "kmsProviders.azure")) {
+         char *azure_tenant_id;
+         char *azure_client_id;
+         char *azure_client_secret;
+
+         azure_tenant_id =
+            test_framework_getenv ("MONGOC_TEST_AZURE_TENANT_ID");
+         azure_client_id =
+            test_framework_getenv ("MONGOC_TEST_AZURE_CLIENT_ID");
+         azure_client_secret =
+            test_framework_getenv ("MONGOC_TEST_AZURE_CLIENT_SECRET");
+
+         if (!azure_tenant_id || !azure_client_id || !azure_client_secret) {
+            fprintf (stderr,
+                     "Set MONGOC_TEST_AZURE_TENANT_ID, "
+                     "MONGOC_TEST_AZURE_CLIENT_ID, and "
+                     "MONGOC_TEST_AZURE_CLIENT_SECRET to enable CSFLE "
+                     "tests.");
+            abort ();
+         }
+
+         bson_concat (&kms_providers,
+                      tmp_bson ("{ 'azure': { 'tenantId': '%s', 'clientId': "
+                                "'%s', 'clientSecret': '%s' }}",
+                                azure_tenant_id,
+                                azure_client_id,
+                                azure_client_secret));
+
+         bson_free (azure_tenant_id);
+         bson_free (azure_client_id);
+         bson_free (azure_client_secret);
+      }
+
+      if (bson_has_field (&opts, "kmsProviders.gcp")) {
+         char *gcp_email;
+         char *gcp_privatekey;
+
+         gcp_email = test_framework_getenv ("MONGOC_TEST_GCP_EMAIL");
+         gcp_privatekey = test_framework_getenv ("MONGOC_TEST_GCP_PRIVATEKEY");
+
+         if (!gcp_email || !gcp_privatekey) {
+            fprintf (stderr,
+                     "Set MONGOC_TEST_GCP_EMAIL and "
+                     "MONGOC_TEST_GCP_PRIVATEKEY to enable CSFLE "
+                     "tests.");
+            abort ();
+         }
+
+         bson_concat (
+            &kms_providers,
+            tmp_bson ("{ 'gcp': { 'email': '%s', 'privateKey': '%s' }}",
+                      gcp_email,
+                      gcp_privatekey));
+
+         bson_free (gcp_email);
+         bson_free (gcp_privatekey);
+      }
+
       mongoc_auto_encryption_opts_set_kms_providers (auto_encryption_opts,
                                                      &kms_providers);
       bson_destroy (&kms_providers);
@@ -1508,13 +1573,13 @@ set_auto_encryption_opts (mongoc_client_t *client, bson_t *test)
 }
 
 static bool
-_in_blacklist (const bson_t *test, char **blacklist, uint32_t blacklist_len)
+_in_deny_list (const bson_t *test, char **deny_list, uint32_t deny_list_len)
 {
    int i;
    const char *desc = bson_lookup_utf8 (test, "description");
 
-   for (i = 0; i < blacklist_len; i++) {
-      if (0 == strcmp (desc, blacklist[i])) {
+   for (i = 0; i < deny_list_len; i++) {
+      if (0 == strcmp (desc, deny_list[i])) {
          return true;
       }
    }
@@ -1525,10 +1590,10 @@ _in_blacklist (const bson_t *test, char **blacklist, uint32_t blacklist_len)
 static bool
 _should_skip_due_to_unsupported_operation (const bson_t *test)
 {
-   char *blacklist[] = {"CreateIndex and dropIndex omits default write concern",
+   char *deny_list[] = {"CreateIndex and dropIndex omits default write concern",
                         "MapReduce omits default write concern"};
 
-   if (_in_blacklist (test, blacklist, sizeof (blacklist) / sizeof (char *))) {
+   if (_in_deny_list (test, deny_list, sizeof (deny_list) / sizeof (char *))) {
       return true;
    }
 
@@ -1629,7 +1694,8 @@ run_json_general_test (const json_test_config_t *config)
 
       bson_free (selected_test);
 
-      uri = test_framework_get_uri ();
+      uri = (config->uri_str != NULL) ? mongoc_uri_new (config->uri_str)
+                                      : test_framework_get_uri ();
 
       /* If we are using multiple mongos, hardcode them in, for now, but keep
        * the other URI components (CDRIVER-3285) */
@@ -1651,7 +1717,7 @@ run_json_general_test (const json_test_config_t *config)
          set_uri_opts_from_bson (uri, &client_opts);
       }
 
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
       mongoc_client_set_error_api (client, MONGOC_ERROR_API_VERSION_2);
       test_framework_set_ssl_opts (client);
       /* reconnect right away, if a fail point causes a disconnect */
@@ -1671,9 +1737,12 @@ run_json_general_test (const json_test_config_t *config)
                                            NULL,
                                            &error);
 
-      /* expect "operation was interrupted", ignore "command not found" */
-      if (!r && (error.domain != MONGOC_ERROR_SERVER ||
-                 (error.code != 11601 && error.code != 59))) {
+      /* expect "operation was interrupted", ignore "command not found" or "is
+       * not supported" */
+      if (!r &&
+          (error.domain != MONGOC_ERROR_SERVER ||
+           (error.code != 11601 && error.code != 59)) &&
+          (strstr (error.message, "is unsupported") == NULL)) {
          MONGOC_WARNING ("Error in killAllSessions: %s", error.message);
       }
 
