@@ -26,7 +26,7 @@
 #include "json-test-monitoring.h"
 #include "json-test-operations.h"
 
-#define MAX_NUM_TESTS 100
+#define MAX_NUM_TESTS 150
 
 typedef void (*test_hook) (bson_t *test);
 
@@ -38,11 +38,13 @@ typedef struct _json_test_config_t {
    json_test_events_check_cb_t events_check_cb;
    bool command_started_events_only;
    bool command_monitoring_allow_subset;
+   const char *uri_str;
 } json_test_config_t;
 
-#define JSON_TEST_CONFIG_INIT       \
-   {                                \
-      NULL, NULL, NULL, NULL, false \
+
+#define JSON_TEST_CONFIG_INIT                                \
+   {                                                         \
+      NULL, NULL, NULL, NULL, NULL, NULL, false, false, NULL \
    }
 
 bson_t *
@@ -67,7 +69,7 @@ server_description_by_hostname (mongoc_topology_description_t *topology,
                                 const char *address);
 
 void
-process_sdam_test_ismaster_responses (bson_t *phase,
+process_sdam_test_hello_responses (bson_t *phase,
                                       mongoc_topology_t *topology);
 
 void
