@@ -109,7 +109,7 @@ make_uri (mongoc_array_t *servers)
 
 
 static char *
-ismaster_json (mock_rs_t *rs,
+hello_json (mock_rs_t *rs,
                mongoc_server_description_type_t type,
                const bson_t *tags)
 {
@@ -120,12 +120,12 @@ ismaster_json (mock_rs_t *rs,
    char *json;
 
    if (type == MONGOC_SERVER_RS_PRIMARY) {
-      server_type = "'ismaster': true, 'secondary': false, ";
+      server_type = "'isWritablePrimary': true, 'secondary': false, ";
    } else if (type == MONGOC_SERVER_RS_SECONDARY) {
-      server_type = "'ismaster': false, 'secondary': true, ";
+      server_type = "'isWritablePrimary': false, 'secondary': true, ";
    } else {
       BSON_ASSERT (type == MONGOC_SERVER_RS_ARBITER);
-      server_type = "'ismaster': false, 'arbiterOnly': true, ";
+      server_type = "'isWritablePrimary': false, 'arbiterOnly': true, ";
    }
 
    if (rs->max_wire_version >= WIRE_VERSION_OP_MSG) {
@@ -168,14 +168,14 @@ ismaster_json (mock_rs_t *rs,
 static char *
 primary_json (mock_rs_t *rs)
 {
-   return ismaster_json (rs, MONGOC_SERVER_RS_PRIMARY, &rs->primary_tags);
+   return hello_json (rs, MONGOC_SERVER_RS_PRIMARY, &rs->primary_tags);
 }
 
 
 static char *
 secondary_json (mock_rs_t *rs, int server_number)
 {
-   return ismaster_json (
+   return hello_json (
       rs, MONGOC_SERVER_RS_SECONDARY, rs->secondary_tags[server_number]);
 }
 
@@ -183,15 +183,15 @@ secondary_json (mock_rs_t *rs, int server_number)
 static char *
 arbiter_json (mock_rs_t *rs)
 {
-   return ismaster_json (rs, MONGOC_SERVER_RS_ARBITER, NULL);
+   return hello_json (rs, MONGOC_SERVER_RS_ARBITER, NULL);
 }
 
 
 /*--------------------------------------------------------------------------
  *
- * mock_rs_with_autoismaster --
+ * mock_rs_with_auto_hello --
  *
- *       A new mock replica set. Each member autoresponds to ismaster.
+ *       A new mock replica set. Each member autoresponds to hello.
  *       Call mock_rs_run to start it, then mock_rs_get_uri to connect.
  *
  * Returns:
@@ -204,10 +204,10 @@ arbiter_json (mock_rs_t *rs)
  */
 
 mock_rs_t *
-mock_rs_with_autoismaster (int32_t max_wire_version,
-                           bool has_primary,
-                           int n_secondaries,
-                           int n_arbiters)
+mock_rs_with_auto_hello (int32_t max_wire_version,
+                         bool has_primary,
+                         int n_secondaries,
+                         int n_arbiters)
 {
    int i;
    mock_rs_t *rs = (mock_rs_t *) bson_malloc0 (sizeof (mock_rs_t));
@@ -322,7 +322,7 @@ mock_rs_run (mock_rs_t *rs)
    int i;
    mock_server_t *server;
    char *hosts_str;
-   char *ismaster_reply;
+   char *hello;
 
    if (rs->has_primary) {
       /* start primary */
@@ -359,7 +359,7 @@ mock_rs_run (mock_rs_t *rs)
 
    /* enqueue unhandled requests in rs->q, they're retrieved with
     * mock_rs_receives_query() &co. rs_q_append is added first so it
-    * runs last, after auto_ismaster.
+    * runs last, after auto_hello.
     */
    for (i = 0; i < rs->servers.len; i++) {
       mock_server_autoresponds (
@@ -373,28 +373,27 @@ mock_rs_run (mock_rs_t *rs)
 
    BSON_ASSERT (rs->max_wire_version > 0);
    if (rs->has_primary) {
-      /* primary's ismaster response */
-      ismaster_reply = primary_json (rs);
-      mock_server_auto_ismaster (rs->primary, ismaster_reply);
-      bson_free (ismaster_reply);
+      /* primary's hello response */
+      hello = primary_json (rs);
+      mock_server_auto_hello (rs->primary, hello);
+      bson_free (hello);
    }
 
-   /* secondaries' ismaster response */
+   /* secondaries' hello response */
    for (i = 0; i < rs->n_secondaries; i++) {
-      ismaster_reply = secondary_json (rs, i);
-      mock_server_auto_ismaster (get_server (&rs->secondaries, i),
-                                 ismaster_reply);
-      bson_free (ismaster_reply);
+      hello = secondary_json (rs, i);
+      mock_server_auto_hello (get_server (&rs->secondaries, i), hello);
+      bson_free (hello);
    }
 
-   /* arbiters' ismaster response */
-   ismaster_reply = arbiter_json (rs);
+   /* arbiters' hello response */
+   hello = arbiter_json (rs);
 
    for (i = 0; i < rs->n_arbiters; i++) {
-      mock_server_auto_ismaster (get_server (&rs->arbiters, i), ismaster_reply);
+      mock_server_auto_hello (get_server (&rs->arbiters, i), hello);
    }
 
-   bson_free (ismaster_reply);
+   bson_free (hello);
 
    if (rs->max_wire_version >= WIRE_VERSION_OP_MSG) {
       mock_rs_auto_endsessions (rs);
@@ -935,7 +934,7 @@ mock_rs_stepdown (mock_rs_t *rs)
    bson_reinit (&rs->primary_tags);
 
    json = secondary_json (rs, rs->n_secondaries - 1);
-   mock_server_auto_ismaster (rs->primary, json);
+   mock_server_auto_hello (rs->primary, json);
    bson_free (json);
 
    _mongoc_array_append_val (&rs->secondaries, rs->primary);
@@ -971,7 +970,7 @@ mock_rs_elect (mock_rs_t *rs, int id)
 
    /* primary_json() uses the current primary_tags */
    json = primary_json (rs);
-   mock_server_auto_ismaster (rs->primary, json);
+   mock_server_auto_hello (rs->primary, json);
    bson_free (json);
 
    ptrs = (mock_server_t **) rs->secondaries.data;
