@@ -53,10 +53,10 @@ get_localhost_stream (uint16_t port)
 
 
 static void
-test_ismaster_helper (mongoc_async_cmd_t *acmd,
-                      mongoc_async_cmd_result_t result,
-                      const bson_t *bson,
-                      int64_t duration_usec)
+test_hello_helper (mongoc_async_cmd_t *acmd,
+                   mongoc_async_cmd_result_t result,
+                   const bson_t *bson,
+                   int64_t duration_usec)
 {
    struct result *r = (struct result *) acmd->data;
    bson_iter_t iter;
@@ -80,7 +80,7 @@ test_ismaster_helper (mongoc_async_cmd_t *acmd,
 
 
 static void
-test_ismaster_impl (bool with_ssl)
+test_hello_impl (bool with_ssl)
 {
    mock_server_t *servers[NSERVERS];
    mongoc_async_t *async;
@@ -106,7 +106,7 @@ test_ismaster_impl (bool with_ssl)
       return;
    }
 
-   BSON_ASSERT (bson_append_int32 (&q, "isMaster", 8, 1));
+   BSON_ASSERT (BSON_APPEND_INT32 (&q, HANDSHAKE_CMD_LEGACY_HELLO, 1));
 
    for (i = 0; i < NSERVERS; i++) {
       servers[i] = mock_server_new ();
@@ -153,7 +153,7 @@ test_ismaster_impl (bool with_ssl)
                             setup_ctx,
                             "admin",
                             &q,
-                            &test_ismaster_helper,
+                            &test_hello_helper,
                             (void *) &results[i],
                             TIMEOUT);
    }
@@ -165,11 +165,11 @@ test_ismaster_impl (bool with_ssl)
    for (i = 0; i < NSERVERS; i++) {
       server_id = (i + offset) % NSERVERS;
       request = mock_server_receives_command (
-         servers[server_id], "admin", MONGOC_QUERY_SLAVE_OK, NULL);
+         servers[server_id], "admin", MONGOC_QUERY_SECONDARY_OK, NULL);
 
       /* use "serverId" field to distinguish among responses */
       reply = bson_strdup_printf ("{'ok': 1,"
-                                  " 'ismaster': true,"
+                                  " '"HANDSHAKE_RESPONSE_LEGACY_HELLO"': true,"
                                   " 'minWireVersion': 0,"
                                   " 'maxWireVersion': 1000,"
                                   " 'serverId': %d}",
@@ -202,22 +202,22 @@ test_ismaster_impl (bool with_ssl)
 }
 
 static void
-test_ismaster (void)
+test_hello (void)
 {
-   test_ismaster_impl (false);
+   test_hello_impl (false);
 }
 
 
 #if defined(MONGOC_ENABLE_SSL_OPENSSL)
 static void
-test_ismaster_ssl (void)
+test_hello_ssl (void)
 {
-   test_ismaster_impl (true);
+   test_hello_impl (true);
 }
 #else
 
 static void
-test_large_ismaster_helper (mongoc_async_cmd_t *acmd,
+test_large_hello_helper (mongoc_async_cmd_t *acmd,
                             mongoc_async_cmd_result_t result,
                             const bson_t *bson,
                             int64_t duration_usec)
@@ -235,31 +235,32 @@ test_large_ismaster_helper (mongoc_async_cmd_t *acmd,
    }
    ASSERT_CMPINT (result, ==, MONGOC_ASYNC_CMD_SUCCESS);
 
-   BSON_ASSERT (bson_iter_init_find (&iter, bson, "ismaster"));
+   ASSERT_HAS_FIELD (bson, HANDSHAKE_RESPONSE_LEGACY_HELLO);
+   BSON_ASSERT (bson_iter_init_find (&iter, bson, HANDSHAKE_RESPONSE_LEGACY_HELLO));
    BSON_ASSERT (BSON_ITER_HOLDS_BOOL (&iter) && bson_iter_bool (&iter));
 }
 
 static void
-test_large_ismaster (void *ctx)
+test_large_hello (void *ctx)
 {
    mongoc_async_t *async;
    mongoc_stream_t *sock_stream;
-   int i = 0;
    bson_t q = BSON_INITIALIZER;
+   char buf[1024 * 1024];
+   mongoc_server_api_t *default_api = NULL;
 
 #ifdef MONGOC_ENABLE_SSL
    mongoc_ssl_opt_t ssl_opts;
 #endif
 
-   /* Inflate the size of the isMaster message with other fields to ~1MB.
-    * mongod should ignore them, but this tests that CDRIVER-2483 is fixed.
+   /* Inflate the size of the hello message to ~1MB. This tests that
+    * CDRIVER-2483 is fixed. Because mongod 4.9+ errors on unknown and duplicate
+    * fields (see SERVER-53150) we add a ~1MB comment.
     */
-   BSON_ASSERT (bson_append_int32 (&q, "isMaster", 8, 1));
-   while (q.len < 1024 * 1024) {
-      char buf[11];
-      bson_snprintf (buf, sizeof (buf), "key_%06d", i++);
-      BSON_APPEND_INT32 (&q, buf, 0);
-   }
+   BSON_ASSERT (BSON_APPEND_INT32 (&q, HANDSHAKE_CMD_LEGACY_HELLO, 1));
+   /* size of comment string = (1024 * 1024) - 1 (for null terminator) */
+   bson_snprintf (buf, sizeof (buf), "%01048575d", 0);
+   BSON_APPEND_UTF8 (&q, "comment", buf);
 
    sock_stream = get_localhost_stream (test_framework_get_port ());
 
@@ -271,6 +272,12 @@ test_large_ismaster (void *ctx)
    }
 #endif
 
+   default_api = test_framework_get_default_server_api ();
+   if (default_api) {
+      _mongoc_cmd_append_server_api (&q, default_api);
+   }
+   mongoc_server_api_destroy (default_api);
+
    async = mongoc_async_new ();
    mongoc_async_cmd_new (async,
                          sock_stream,
@@ -287,7 +294,7 @@ test_large_ismaster (void *ctx)
                          NULL,
                          "admin",
                          &q,
-                         &test_large_ismaster_helper,
+                         &test_large_hello_helper,
                          NULL,
                          TIMEOUT);
 
@@ -304,27 +311,27 @@ typedef struct _stream_with_result_t {
 } stream_with_result_t;
 
 static void
-test_ismaster_delay_callback (mongoc_async_cmd_t *acmd,
-                              mongoc_async_cmd_result_t result,
-                              const bson_t *bson,
-                              int64_t duration_usec)
+test_hello_delay_callback (mongoc_async_cmd_t *acmd,
+                           mongoc_async_cmd_result_t result,
+                           const bson_t *bson,
+                           int64_t duration_usec)
 {
    ((stream_with_result_t *) acmd->data)->finished = true;
 }
 
 static mongoc_stream_t *
-test_ismaster_delay_initializer (mongoc_async_cmd_t *acmd)
+test_hello_delay_initializer (mongoc_async_cmd_t *acmd)
 {
    return ((stream_with_result_t *) acmd->data)->stream;
 }
 
 static void
-test_ismaster_delay ()
+test_hello_delay ()
 {
    /* test that a delayed cmd works. */
-   mock_server_t *server = mock_server_with_autoismaster (WIRE_VERSION_MAX);
+   mock_server_t *server = mock_server_with_auto_hello (WIRE_VERSION_MAX);
    mongoc_async_t *async = mongoc_async_new ();
-   bson_t ismaster_cmd = BSON_INITIALIZER;
+   bson_t hello_cmd = BSON_INITIALIZER;
    stream_with_result_t stream_with_result = {0};
    int64_t start = bson_get_monotonic_time ();
 
@@ -334,18 +341,18 @@ test_ismaster_delay ()
       get_localhost_stream (mock_server_get_port (server));
    stream_with_result.finished = false;
 
-   BSON_ASSERT (bson_append_int32 (&ismaster_cmd, "isMaster", 8, 1));
+   BSON_ASSERT (BSON_APPEND_INT32 (&hello_cmd, HANDSHAKE_CMD_LEGACY_HELLO, 1));
    mongoc_async_cmd_new (async,
                          NULL,  /* stream, initialized after delay. */
                          false, /* is setup done. */
                          NULL,  /* dns result. */
-                         test_ismaster_delay_initializer,
+                         test_hello_delay_initializer,
                          100,  /* delay 100ms. */
                          NULL, /* setup function. */
                          NULL, /* setup ctx. */
                          "admin",
-                         &ismaster_cmd,
-                         &test_ismaster_delay_callback,
+                         &hello_cmd,
+                         &test_hello_delay_callback,
                          &stream_with_result,
                          TIMEOUT);
 
@@ -356,7 +363,7 @@ test_ismaster_delay ()
       bson_get_monotonic_time () - start, >, (int64_t) (100 * 1000));
    BSON_ASSERT (stream_with_result.finished);
 
-   bson_destroy (&ismaster_cmd);
+   bson_destroy (&hello_cmd);
    mongoc_stream_destroy (stream_with_result.stream);
    mongoc_async_destroy (async);
    mock_server_destroy (server);
@@ -365,20 +372,19 @@ test_ismaster_delay ()
 void
 test_async_install (TestSuite *suite)
 {
-   TestSuite_AddMockServerTest (suite, "/Async/ismaster", test_ismaster);
+   TestSuite_AddMockServerTest (suite, "/Async/hello", test_hello);
 #if defined(MONGOC_ENABLE_SSL_OPENSSL)
-   TestSuite_AddMockServerTest (
-      suite, "/Async/ismaster_ssl", test_ismaster_ssl);
+   TestSuite_AddMockServerTest (suite, "/Async/hello_ssl", test_hello_ssl);
 #else
    /* Skip this test on OpenSSL since was having issues connecting. */
    /* Skip on Windows until CDRIVER-3519 is resolved. */
    TestSuite_AddFull (suite,
-                      "/Async/large_ismaster",
-                      test_large_ismaster,
+                      "/Async/large_hello",
+                      test_large_hello,
                       NULL /* dtor */,
                       NULL /* ctx */,
                       test_framework_skip_if_not_single,
                       test_framework_skip_if_windows);
 #endif
-   TestSuite_AddMockServerTest (suite, "/Async/delay", test_ismaster_delay);
+   TestSuite_AddMockServerTest (suite, "/Async/delay", test_hello_delay);
 }
