@@ -47,7 +47,7 @@ test_get_max_bson_obj_size (void)
    uint32_t id;
 
    /* single-threaded */
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    BSON_ASSERT (client);
 
    id = server_id_for_reads (&client->cluster);
@@ -60,7 +60,7 @@ test_get_max_bson_obj_size (void)
    mongoc_client_destroy (client);
 
    /* multi-threaded */
-   pool = test_framework_client_pool_new ();
+   pool = test_framework_new_default_client_pool ();
    client = mongoc_client_pool_pop (pool);
 
    id = server_id_for_reads (&client->cluster);
@@ -84,7 +84,7 @@ test_get_max_msg_size (void)
    uint32_t id;
 
    /* single-threaded */
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    id = server_id_for_reads (&client->cluster);
 
    sd = (mongoc_server_description_t *) mongoc_set_get (
@@ -96,7 +96,7 @@ test_get_max_msg_size (void)
    mongoc_client_destroy (client);
 
    /* multi-threaded */
-   pool = test_framework_client_pool_new ();
+   pool = test_framework_new_default_client_pool ();
    client = mongoc_client_pool_pop (pool);
 
    id = server_id_for_reads (&client->cluster);
@@ -128,7 +128,7 @@ test_get_max_msg_size (void)
          collection, tmp_bson ("{}"), NULL, NULL);                      \
       future = future_cursor_next (cursor, &doc);                       \
       request = mock_server_receives_query (                            \
-         server, "test.test", MONGOC_QUERY_SLAVE_OK, 0, 0, "{}", NULL); \
+         server, "test.test", MONGOC_QUERY_SECONDARY_OK, 0, 0, "{}", NULL); \
       BSON_ASSERT (request);                                            \
       client_port_variable = request_get_client_port (request);         \
    } while (0)
@@ -164,16 +164,16 @@ _test_cluster_node_disconnect (bool pooled)
 
    capture_logs (true);
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_MIN);
+   server = mock_server_with_auto_hello (WIRE_VERSION_MIN);
    mock_server_run (server);
 
    uri = mongoc_uri_copy (mock_server_get_uri (server));
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
    }
 
    collection = mongoc_client_get_collection (client, "test", "test");
@@ -238,23 +238,23 @@ _test_cluster_command_timeout (bool pooled)
 
    capture_logs (true);
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_MIN);
+   server = mock_server_with_auto_hello (WIRE_VERSION_MIN);
    mock_server_run (server);
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    mongoc_uri_set_option_as_int32 (uri, "socketTimeoutMS", 200);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
    }
 
    /* server doesn't respond in time */
    future = future_client_command_simple (
       client, "db", tmp_bson ("{'foo': 1}"), NULL, NULL, &error);
    request =
-      mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+      mock_server_receives_command (server, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
    client_port = request_get_client_port (request);
 
    ASSERT (!future_get_bool (future));
@@ -277,7 +277,7 @@ _test_cluster_command_timeout (bool pooled)
    future = future_client_command_simple (
       client, "db", tmp_bson ("{'baz': 1}"), NULL, &reply, &error);
    request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK, "{'baz': 1}");
+      server, "db", MONGOC_QUERY_SECONDARY_OK, "{'baz': 1}");
    ASSERT (request);
    /* new socket */
    ASSERT_CMPUINT16 (client_port, !=, request_get_client_port (request));
@@ -320,7 +320,7 @@ static void
 _test_write_disconnect (void)
 {
    mock_server_t *server;
-   char *ismaster_response;
+   char *hello;
    mongoc_client_t *client;
    mongoc_collection_t *collection;
    bson_error_t error;
@@ -335,24 +335,25 @@ _test_write_disconnect (void)
 
    server = mock_server_new ();
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
 
    /*
-    * establish connection with an "ismaster" and "ping"
+    * establish connection with an "hello" and "ping"
     */
    future = future_client_command_simple (
       client, "db", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
-   request = mock_server_receives_ismaster (server);
-   ismaster_response = bson_strdup_printf ("{'ok': 1.0,"
-                                           " 'ismaster': true,"
-                                           " 'minWireVersion': 2,"
-                                           " 'maxWireVersion': 3}");
+   request = mock_server_receives_legacy_hello (server, NULL);
+   hello = bson_strdup_printf ("{'ok': 1.0,"
+                               " 'isWritablePrimary': true,"
+                               " 'minWireVersion': 2,"
+                               " 'maxWireVersion': 3}");
 
-   mock_server_replies_simple (request, ismaster_response);
+   mock_server_replies_simple (request, hello);
    request_destroy (request);
 
    request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK, "{'ping': 1}");
+      server, "db", MONGOC_QUERY_SECONDARY_OK, "{'ping': 1}");
    mock_server_replies_simple (request, "{'ok': 1}");
    ASSERT_OR_PRINT (future_get_bool (future), error);
 
@@ -385,7 +386,7 @@ _test_write_disconnect (void)
    mongoc_collection_destroy (collection);
    request_destroy (request);
    future_destroy (future);
-   bson_free (ismaster_response);
+   bson_free (hello);
    mongoc_client_destroy (client);
    mock_server_destroy (server);
 }
@@ -401,7 +402,7 @@ test_write_command_disconnect (void *ctx)
 /* Test for CDRIVER-3306 - Do not assume non-empty server reply implies stream
  * is still valid */
 static void
-test_cluster_command_notmaster (void)
+test_cluster_command_not_primary (void)
 {
    mock_server_t *server;
    mongoc_uri_t *uri;
@@ -422,21 +423,21 @@ test_cluster_command_notmaster (void)
    server = mock_server_new ();
    mock_server_run (server);
 
-   /* server is "recovering": not master, not secondary */
-   mock_server_auto_ismaster (server,
-                              "{'ok': 1,"
-                              " 'maxWireVersion': %d,"
-                              " 'ismaster': false,"
-                              " 'secondary': false,"
-                              " 'setName': 'rs',"
-                              " 'hosts': ['%s']}",
-                              WIRE_VERSION_OP_MSG - 1,
-                              mock_server_get_host_and_port (server));
+   /* server is "recovering": not primary, not secondary */
+   mock_server_auto_hello (server,
+                           "{'ok': 1,"
+                           " 'maxWireVersion': %d,"
+                           " 'isWritablePrimary': false,"
+                           " 'secondary': false,"
+                           " 'setName': 'rs',"
+                           " 'hosts': ['%s']}",
+                           WIRE_VERSION_OP_MSG - 1,
+                           mock_server_get_host_and_port (server));
 
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    mongoc_uri_set_option_as_utf8 (uri, "replicaSet", "rs");
 
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
 
    collection = mongoc_client_get_collection (client, "db", "test");
    /* use an unordered bulk write, so it attempts to continue on error */
@@ -458,7 +459,7 @@ test_cluster_command_notmaster (void)
 
    request = mock_server_receives_request (server);
    mock_server_replies_simple (
-      request, "{ 'code': 10107, 'errmsg': 'not master', 'ok': 0 }");
+      request, "{ 'code': 10107, 'errmsg': 'not primary', 'ok': 0 }");
    ASSERT (future_wait (future));
 
    mongoc_client_destroy (client);
@@ -578,7 +579,7 @@ _test_cluster_time (bool pooled, command_fn_t command)
                                         test_cluster_time_cmd_succeeded_cb);
 
    if (pooled) {
-      pool = test_framework_client_pool_new ();
+      pool = test_framework_new_default_client_pool ();
       mongoc_client_pool_set_apm_callbacks (
          pool, callbacks, &cluster_time_test);
       client = mongoc_client_pool_pop (pool);
@@ -586,7 +587,7 @@ _test_cluster_time (bool pooled, command_fn_t command)
        * the test operations. */
       _mongoc_usleep (5000 * 1000); /* 5 s */
    } else {
-      client = test_framework_client_new ();
+      client = test_framework_new_default_client ();
       mongoc_client_set_apm_callbacks (client, callbacks, &cluster_time_test);
    }
 
@@ -899,8 +900,8 @@ future_ping (mongoc_client_t *client, bson_error_t *error)
 static void
 _test_cluster_time_comparison (bool pooled)
 {
-   const char *ismaster =
-      "{'ok': 1.0, 'ismaster': true, 'msg': 'isdbgrid', 'maxWireVersion': 6}";
+   const char *hello = "{'ok': 1.0, 'isWritablePrimary': true, 'msg': "
+                       "'isdbgrid', 'maxWireVersion': 6}";
    mock_server_t *server;
    mongoc_uri_t *uri;
    mongoc_client_pool_t *pool = NULL;
@@ -916,22 +917,22 @@ _test_cluster_time_comparison (bool pooled)
    mongoc_uri_set_option_as_int32 (uri, "heartbeatFrequencyMS", 500);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
    }
 
    future = future_ping (client, &error);
 
    /* timestamp is 1 */
-   request = mock_server_receives_ismaster (server);
-   replies_with_cluster_time (request, 1, 1, ismaster);
+   request = mock_server_receives_legacy_hello (server, NULL);
+   replies_with_cluster_time (request, 1, 1, hello);
 
    if (pooled) {
       /* a pooled client handshakes its own connection */
-      request = mock_server_receives_ismaster (server);
-      replies_with_cluster_time (request, 1, 1, ismaster);
+      request = mock_server_receives_legacy_hello (server, NULL);
+      replies_with_cluster_time (request, 1, 1, hello);
    }
 
    request = receives_with_cluster_time (server, 1, 1, ping);
@@ -956,14 +957,13 @@ _test_cluster_time_comparison (bool pooled)
 
    if (pooled) {
       /* wait for next heartbeat, it should contain newest cluster time */
-      request = mock_server_receives_command (server,
-                                              "admin",
-                                              MONGOC_QUERY_SLAVE_OK,
-                                              "{'isMaster': 1, '$clusterTime': "
-                                              "{'clusterTime': {'$timestamp': "
-                                              "{'t': 2, 'i': 2}}}}");
+      request =
+         mock_server_receives_legacy_hello (server,
+                                            "{'$clusterTime': "
+                                            "{'clusterTime': {'$timestamp': "
+                                            "{'t': 2, 'i': 2}}}}");
 
-      replies_with_cluster_time (request, 2, 1, ismaster);
+      replies_with_cluster_time (request, 2, 1, hello);
 
       mongoc_client_pool_push (pool, client);
       mongoc_client_pool_destroy (pool);
@@ -971,14 +971,13 @@ _test_cluster_time_comparison (bool pooled)
       /* trigger next heartbeat, it should contain newest cluster time */
       _mongoc_usleep (750 * 1000); /* 750 ms */
       future = future_ping (client, &error);
-      request = mock_server_receives_command (server,
-                                              "admin",
-                                              MONGOC_QUERY_SLAVE_OK,
-                                              "{'isMaster': 1, '$clusterTime': "
-                                              "{'clusterTime': {'$timestamp': "
-                                              "{'t': 2, 'i': 2}}}}");
+      request =
+         mock_server_receives_legacy_hello (server,
+                                            "{'$clusterTime': "
+                                            "{'clusterTime': {'$timestamp': "
+                                            "{'t': 2, 'i': 2}}}}");
 
-      replies_with_cluster_time (request, 2, 1, ismaster);
+      replies_with_cluster_time (request, 2, 1, hello);
       request = receives_with_cluster_time (server, 2, 2, ping);
       mock_server_replies_ok_and_destroys (request);
       assert_ok (future, &error);
@@ -1011,7 +1010,7 @@ typedef void (*cleanup_fn_t) (future_t *);
 
 typedef struct {
    const char *errmsg;
-   bool is_not_master_err;
+   bool is_not_primary_err;
 } test_error_msg_t;
 
 
@@ -1019,21 +1018,21 @@ test_error_msg_t errors[] = {
    {"not master", true},
    {"not master or secondary", true},
    {"node is recovering", true},
-   {"not master and slaveOk=false", true},
+   {"not master and secondaryOk=false", true},
    {"replicatedToNum called but not master anymore", true},
    {"??? node is recovering ???", true},
    {"??? not master ???", true},
    {"foo", false},
    {0}};
 
-/* a "not master" or "node is recovering" error marks server Unknown.
-   "not master" and "node is recovering" need only be substrings of the error
+/* a "not primary" or "node is recovering" error marks server Unknown.
+   "not primary" and "node is recovering" need only be substrings of the error
    message. */
 static void
-_test_not_master (bool pooled,
-                  bool use_op_msg,
-                  run_command_fn_t run_command,
-                  cleanup_fn_t cleanup_fn)
+_test_not_primary (bool pooled,
+                   bool use_op_msg,
+                   run_command_fn_t run_command,
+                   cleanup_fn_t cleanup_fn)
 {
    test_error_msg_t *test_error_msg;
    mock_server_t *server;
@@ -1047,14 +1046,16 @@ _test_not_master (bool pooled,
    mongoc_server_description_t *sd;
    char *reply;
 
-   server = mock_server_with_autoismaster (use_op_msg ? 6 : 5);
+   server = mock_server_with_auto_hello (use_op_msg ? 6 : 5);
    mock_server_run (server);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (mock_server_get_uri (server));
+      pool = test_framework_client_pool_new_from_uri (
+         mock_server_get_uri (server), NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+      client = test_framework_client_new_from_uri (mock_server_get_uri (server),
+                                                   NULL);
    }
 
    td = &client->topology->description;
@@ -1075,7 +1076,7 @@ _test_not_master (bool pooled,
       BSON_ASSERT (sd->type == MONGOC_SERVER_STANDALONE);
 
       /*
-       * command error marks server Unknown iff it's a "not master" error
+       * command error marks server Unknown if it's a "not primary" error
        */
       future = run_command (client);
       request = mock_server_receives_request (server);
@@ -1085,7 +1086,7 @@ _test_not_master (bool pooled,
       mock_server_replies_simple (request, reply);
       BSON_ASSERT (!future_get_bool (future));
 
-      if (test_error_msg->is_not_master_err) {
+      if (test_error_msg->is_not_primary_err) {
          BSON_ASSERT (sd->type == MONGOC_SERVER_UNKNOWN);
       } else {
          BSON_ASSERT (sd->type == MONGOC_SERVER_STANDALONE);
@@ -1123,32 +1124,32 @@ function_command_simple_cleanup (future_t *future)
 
 
 static void
-test_not_master_single_op_query (void)
+test_not_primary_single_op_query (void)
 {
-   _test_not_master (
+   _test_not_primary (
       false, false, future_command_simple, function_command_simple_cleanup);
 }
 
 
 static void
-test_not_master_pooled_op_query (void)
+test_not_primary_pooled_op_query (void)
 {
-   _test_not_master (
+   _test_not_primary (
       true, false, future_command_simple, function_command_simple_cleanup);
 }
 
 static void
-test_not_master_single_op_msg (void)
+test_not_primary_single_op_msg (void)
 {
-   _test_not_master (
+   _test_not_primary (
       false, true, future_command_simple, function_command_simple_cleanup);
 }
 
 
 static void
-test_not_master_pooled_op_msg (void)
+test_not_primary_pooled_op_msg (void)
 {
-   _test_not_master (
+   _test_not_primary (
       true, true, future_command_simple, function_command_simple_cleanup);
 }
 
@@ -1186,32 +1187,32 @@ future_command_private_cleanup (future_t *future)
 
 
 static void
-test_not_master_auth_single_op_query (void)
+test_not_primary_auth_single_op_query (void)
 {
-   _test_not_master (
+   _test_not_primary (
       false, false, future_command_private, future_command_private_cleanup);
 }
 
 
 static void
-test_not_master_auth_pooled_op_query (void)
+test_not_primary_auth_pooled_op_query (void)
 {
-   _test_not_master (
+   _test_not_primary (
       true, false, future_command_private, future_command_private_cleanup);
 }
 
 static void
-test_not_master_auth_single_op_msg (void)
+test_not_primary_auth_single_op_msg (void)
 {
-   _test_not_master (
+   _test_not_primary (
       false, true, future_command_private, future_command_private_cleanup);
 }
 
 
 static void
-test_not_master_auth_pooled_op_msg (void)
+test_not_primary_auth_pooled_op_msg (void)
 {
-   _test_not_master (
+   _test_not_primary (
       true, true, future_command_private, future_command_private_cleanup);
 }
 
@@ -1226,44 +1227,27 @@ typedef struct {
 
 
 static bool
-auto_ismaster (request_t *request, void *data)
+auto_hello_callback (request_t *request, void *data, bson_t *hello_response)
 {
    dollar_query_test_t *test;
-   const char *cluster_time = "";
-   const char *server_type;
-   char *ismaster;
-
-   if (!request->is_command ||
-       strcasecmp (request->command_name, "ismaster") != 0) {
-      return false;
-   }
+   bson_t cluster_time;
 
    test = (dollar_query_test_t *) data;
 
+   bson_init (hello_response);
+   BSON_APPEND_INT32 (hello_response, "ok", 1);
+   BSON_APPEND_BOOL (hello_response, "isWritablePrimary", !test->secondary);
+   BSON_APPEND_BOOL (hello_response, "secondary", test->secondary);
+   BSON_APPEND_INT32 (hello_response, "minWireVersion", 0);
+   BSON_APPEND_INT32 (hello_response, "maxWireVersion", WIRE_VERSION_OP_MSG);
+   BSON_APPEND_UTF8 (hello_response, "setName", "rs");
+
    if (test->cluster_time) {
-      cluster_time =
-         ", '$clusterTime': {'clusterTime': {'$timestamp': {'t': 1, 'i': 1}}}";
+      BSON_APPEND_DOCUMENT_BEGIN (hello_response, "$clusterTime", &cluster_time);
+      BSON_APPEND_TIMESTAMP (&cluster_time, "clusterTime", 1, 1);
+      bson_append_document_end (hello_response, &cluster_time);
    }
 
-   if (test->secondary) {
-      server_type = ", 'ismaster': false, 'secondary': true";
-   } else {
-      server_type = ", 'ismaster': true, 'secondary': false";
-   }
-
-   ismaster = bson_strdup_printf ("{'ok': 1.0,"
-                                  " 'minWireVersion': 0,"
-                                  " 'maxWireVersion': %d,"
-                                  " 'setName': 'rs' %s %s}",
-                                  WIRE_VERSION_OP_MSG,
-                                  server_type,
-                                  cluster_time);
-
-   mock_server_replies_simple (request, ismaster);
-
-   bson_free (ismaster);
-   request_destroy (request);
-
    return true;
 }
 
@@ -1285,10 +1269,11 @@ _test_dollar_query (void *ctx)
    test = (dollar_query_test_t *) ctx;
 
    server = mock_server_new ();
-   mock_server_autoresponds (server, auto_ismaster, test, NULL);
+   mock_server_auto_hello_callback (server, auto_hello_callback, test, NULL);
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
    if (test->secondary) {
       read_prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
@@ -1433,7 +1418,7 @@ dollar_query_test_t tests[] = {
    {NULL}};
 
 static void
-_test_cluster_ismaster_fails (bool hangup)
+_test_cluster_hello_fails (bool hangup)
 {
    mock_server_t *mock_server;
    mongoc_uri_t *uri;
@@ -1447,12 +1432,12 @@ _test_cluster_ismaster_fails (bool hangup)
 
    mock_server = mock_server_new ();
    autoresponder_id =
-      mock_server_auto_ismaster (mock_server, "{ 'isMaster': 1.0 }");
+      mock_server_auto_hello (mock_server, "{ 'isWritablePrimary': true }");
    mock_server_run (mock_server);
    uri = mongoc_uri_copy (mock_server_get_uri (mock_server));
    /* increase heartbeatFrequencyMS to prevent background server selection. */
    mongoc_uri_set_option_as_int32 (uri, "heartbeatFrequencyMS", 99999);
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
    mongoc_client_pool_set_error_api (pool, 2);
    mongoc_uri_destroy (uri);
    client = mongoc_client_pool_pop (pool);
@@ -1466,11 +1451,11 @@ _test_cluster_ismaster_fails (bool hangup)
    future = future_client_command_simple (
       client, "test", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
    /* the client adds a cluster node, creating a stream to the server, and then
-    * sends an ismaster request. */
-   request = mock_server_receives_ismaster (mock_server);
+    * sends a hello request. */
+   request = mock_server_receives_legacy_hello (mock_server, NULL);
    /* CDRIVER-2576: the server replies with an error, so
-    * _mongoc_stream_run_ismaster returns NULL, which
-    * _mongoc_cluster_run_ismaster must check. */
+    * _mongoc_stream_run_hello returns NULL, which
+    * _mongoc_cluster_run_hello must check. */
 
    if (hangup) {
       capture_logs (true); /* suppress "failed to buffer" warning */
@@ -1493,16 +1478,16 @@ _test_cluster_ismaster_fails (bool hangup)
 }
 
 static void
-test_cluster_ismaster_fails (void)
+test_cluster_hello_fails (void)
 {
-   _test_cluster_ismaster_fails (false);
+   _test_cluster_hello_fails (false);
 }
 
 
 static void
-test_cluster_ismaster_hangup (void)
+test_cluster_hello_hangup (void)
 {
-   _test_cluster_ismaster_fails (true);
+   _test_cluster_hello_fails (true);
 }
 
 static void
@@ -1515,12 +1500,13 @@ _test_cluster_command_error (bool use_op_msg)
    future_t *future;
 
    if (use_op_msg) {
-      server = mock_server_with_autoismaster (WIRE_VERSION_OP_MSG);
+      server = mock_server_with_auto_hello (WIRE_VERSION_OP_MSG);
    } else {
-      server = mock_server_with_autoismaster (WIRE_VERSION_OP_MSG - 1);
+      server = mock_server_with_auto_hello (WIRE_VERSION_OP_MSG - 1);
    }
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    future = future_client_command_simple (client,
                                           "db",
                                           tmp_bson ("{'ping': 1}"),
@@ -1532,7 +1518,7 @@ _test_cluster_command_error (bool use_op_msg)
          server, MONGOC_QUERY_NONE, tmp_bson ("{'ping': 1}"));
    } else {
       request = mock_server_receives_command (
-         server, "db", MONGOC_QUERY_SLAVE_OK, "{'ping': 1}");
+         server, "db", MONGOC_QUERY_SECONDARY_OK, "{'ping': 1}");
    }
    mock_server_hangs_up (request);
    BSON_ASSERT (!future_get_bool (future));
@@ -1585,14 +1571,15 @@ test_advanced_cluster_time_not_sent_to_standalone (void)
 
    server = mock_server_new ();
    mock_server_auto_endsessions (server);
-   mock_server_auto_ismaster (server,
-                              "{'ok': 1.0,"
-                              " 'ismaster': true,"
-                              " 'minWireVersion': 0,"
-                              " 'maxWireVersion': 6,"
-                              " 'logicalSessionTimeoutMinutes': 30}");
+   mock_server_auto_hello (server,
+                           "{'ok': 1.0,"
+                           " 'isWritablePrimary': true,"
+                           " 'minWireVersion': 0,"
+                           " 'maxWireVersion': 6,"
+                           " 'logicalSessionTimeoutMinutes': 30}");
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
 
    cs = mongoc_client_start_session (client, NULL, &error);
    ASSERT_OR_PRINT (cs, error);
@@ -1630,16 +1617,17 @@ test_advanced_cluster_time_not_sent_to_standalone (void)
    mock_server_destroy (server);
 }
 
-/* Responds properly to isMaster, hangs up on serverStatus, and replies {ok:1}
+/* Responds properly to hello, hangs up on serverStatus, and replies {ok:1}
  * to everything else. */
 static bool
 _responder (request_t *req, void *data)
 {
-   char *ismaster;
+   char *hello;
 
-   ismaster = (char *) data;
-   if (0 == strcmp (req->command_name, "isMaster")) {
-      mock_server_replies_simple (req, ismaster);
+   hello = (char *) data;
+   if (0 == strcasecmp (req->command_name, HANDSHAKE_CMD_LEGACY_HELLO) ||
+       0 == strcasecmp (req->command_name, "hello")) {
+      mock_server_replies_simple (req, hello);
       request_destroy (req);
       return true;
    } else if (0 == strcmp (req->command_name, "serverStatus")) {
@@ -1688,7 +1676,7 @@ _initiator_fn (const mongoc_uri_t *uri,
 }
 
 static void
-_test_ismaster_on_unknown (char *ismaster)
+_test_hello_on_unknown (char *hello)
 {
    mock_server_t *mock_server;
    mongoc_client_pool_t *pool;
@@ -1699,15 +1687,15 @@ _test_ismaster_on_unknown (char *ismaster)
 
    mock_server = mock_server_new ();
    mock_server_run (mock_server);
-   mock_server_autoresponds (mock_server, _responder, ismaster, NULL);
+   mock_server_autoresponds (mock_server, _responder, hello, NULL);
 
    uri = mongoc_uri_copy (mock_server_get_uri (mock_server));
 
    /* Add a placeholder additional host, so the topology type can be SHARDED.
-    * The host will get removed on the first failed ismaster. */
+    * The host will get removed on the first failed hello. */
    ret = mongoc_uri_upsert_host (uri, "localhost", 12345, &error);
    ASSERT_OR_PRINT (ret, error);
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
 
    client = mongoc_client_pool_pop (pool);
 
@@ -1715,7 +1703,7 @@ _test_ismaster_on_unknown (char *ismaster)
 
    /* The other client marked the server as unknown after this client selected
     * the server and created a stream, but *before* constructing the initial
-    * ismaster. This reproduces the crash reported in CDRIVER-3404. */
+    * hello. This reproduces the crash reported in CDRIVER-3404. */
    ret = mongoc_client_command_simple (
       client, "db", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
    ASSERT_OR_PRINT (ret, error);
@@ -1728,17 +1716,17 @@ _test_ismaster_on_unknown (char *ismaster)
 }
 
 void
-test_ismaster_on_unknown (void)
+test_hello_on_unknown (void)
 {
    /* Test with pre-OP_MSG to test fix to CDRIVER-3404. */
-   _test_ismaster_on_unknown ("{ 'ok': 1.0, 'ismaster': true, "
-                              "'minWireVersion': 0, 'maxWireVersion': 5, "
-                              "'msg': 'isdbgrid'}");
+   _test_hello_on_unknown ("{ 'ok': 1.0, 'isWritablePrimary': true, "
+                           "'minWireVersion': 0, 'maxWireVersion': 5, "
+                           "'msg': 'isdbgrid'}");
 
    /* Test with OP_MSG. */
-   _test_ismaster_on_unknown ("{ 'ok': 1.0, 'ismaster': true, "
-                              "'minWireVersion': 0, 'maxWireVersion': 8, "
-                              "'msg': 'isdbgrid'}");
+   _test_hello_on_unknown ("{ 'ok': 1.0, 'isWritablePrimary': true, "
+                           "'minWireVersion': 0, 'maxWireVersion': 8, "
+                           "'msg': 'isdbgrid'}");
 }
 
 
@@ -1761,12 +1749,12 @@ _test_cmd_on_unknown_serverid (bool pooled)
    mongoc_uri_set_option_as_int32 (uri, MONGOC_URI_HEARTBEATFREQUENCYMS, 99999);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       test_framework_set_pool_ssl_opts (pool);
       client = mongoc_client_pool_pop (pool);
    } else {
       pool = NULL;
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
       test_framework_set_ssl_opts (client);
    }
 
@@ -1871,7 +1859,7 @@ test_cluster_install (TestSuite *suite)
                                 "/Cluster/command/timeout/pooled",
                                 test_cluster_command_timeout_pooled);
    TestSuite_AddMockServerTest (
-      suite, "/Cluster/command/notmaster", test_cluster_command_notmaster);
+      suite, "/Cluster/command/notprimary", test_cluster_command_not_primary);
    TestSuite_AddFull (suite,
                       "/Cluster/write_command/disconnect",
                       test_write_command_disconnect,
@@ -1928,41 +1916,41 @@ test_cluster_install (TestSuite *suite)
       test_advanced_cluster_time_not_sent_to_standalone,
       test_framework_skip_if_no_crypto);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master/single/op_query",
-                                test_not_master_single_op_query,
+                                "/Cluster/not_primary/single/op_query",
+                                test_not_primary_single_op_query,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master/pooled/op_query",
-                                test_not_master_pooled_op_query,
+                                "/Cluster/not_primary/pooled/op_query",
+                                test_not_primary_pooled_op_query,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master/single/op_msg",
-                                test_not_master_single_op_msg,
+                                "/Cluster/not_primary/single/op_msg",
+                                test_not_primary_single_op_msg,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master/pooled/op_msg",
-                                test_not_master_pooled_op_msg,
+                                "/Cluster/not_primary/pooled/op_msg",
+                                test_not_primary_pooled_op_msg,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master_auth/single/op_query",
-                                test_not_master_auth_single_op_query,
+                                "/Cluster/not_primary_auth/single/op_query",
+                                test_not_primary_auth_single_op_query,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master_auth/pooled/op_query",
-                                test_not_master_auth_pooled_op_query,
+                                "/Cluster/not_primary_auth/pooled/op_query",
+                                test_not_primary_auth_pooled_op_query,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master_auth/single/op_msg",
-                                test_not_master_auth_single_op_msg,
+                                "/Cluster/not_primary_auth/single/op_msg",
+                                test_not_primary_auth_single_op_msg,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (suite,
-                                "/Cluster/not_master_auth/pooled/op_msg",
-                                test_not_master_auth_pooled_op_msg,
+                                "/Cluster/not_primary_auth/pooled/op_msg",
+                                test_not_primary_auth_pooled_op_msg,
                                 test_framework_skip_if_slow);
    TestSuite_AddMockServerTest (
-      suite, "/Cluster/ismaster_fails", test_cluster_ismaster_fails);
+      suite, "/Cluster/hello_fails", test_cluster_hello_fails);
    TestSuite_AddMockServerTest (
-      suite, "/Cluster/ismaster_hangup", test_cluster_ismaster_hangup);
+      suite, "/Cluster/hello_hangup", test_cluster_hello_hangup);
    TestSuite_AddMockServerTest (suite,
                                 "/Cluster/command_error/op_msg",
                                 test_cluster_command_error_op_msg);
@@ -1970,7 +1958,7 @@ test_cluster_install (TestSuite *suite)
                                 "/Cluster/command_error/op_query",
                                 test_cluster_command_error_op_query);
    TestSuite_AddMockServerTest (
-      suite, "/Cluster/ismaster_on_unknown/mock", test_ismaster_on_unknown);
+      suite, "/Cluster/hello_on_unknown/mock", test_hello_on_unknown);
    TestSuite_AddLive (
       suite, "/Cluster/cmd_on_unknown_serverid", test_cmd_on_unknown_serverid);
 }
