@@ -111,7 +111,7 @@ _test_collection_find_live (test_collection_find_t *test_data)
    bson_error_t error;
    mongoc_cursor_t *cursor;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    database = mongoc_client_get_database (client, "test");
    collection_name = gen_collection_name ("test");
    collection = mongoc_database_create_collection (
@@ -195,9 +195,10 @@ _test_collection_op_query_or_find_command (test_collection_find_t *test_data,
       return;
    }
 
-   server = mock_server_with_autoismaster (max_wire_version);
+   server = mock_server_with_auto_hello (max_wire_version);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
    cursor = mongoc_collection_find (collection,
                                     test_data->flags,
@@ -261,7 +262,7 @@ _check_op_query (mock_server_t *server, test_collection_find_t *test_data)
 {
    mongoc_query_flags_t flags;
 
-   flags = test_data->flags | MONGOC_QUERY_SLAVE_OK;
+   flags = test_data->flags | MONGOC_QUERY_SECONDARY_OK;
 
    return mock_server_receives_query (server,
                                       "db.collection",
@@ -312,13 +313,13 @@ _test_collection_op_query (test_collection_find_t *test_data)
 static request_t *
 _check_find_command (mock_server_t *server, test_collection_find_t *test_data)
 {
-   /* Server Selection Spec: all queries to standalone set slaveOk.
+   /* Server Selection Spec: all queries to standalone set secondaryOk.
     *
     * Find, getMore And killCursors Commands Spec: "When sending a find command
-    * rather than a legacy OP_QUERY find only the slaveOk flag is honored".
+    * rather than a legacy OP_QUERY find only the secondaryOk flag is honored".
     */
    return mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK, test_data->expected_find_command);
+      server, "db", MONGOC_QUERY_SECONDARY_OK, test_data->expected_find_command);
 }
 
 
@@ -771,7 +772,7 @@ test_query_flags (void)
       const char *json_fragment;
    } flag_and_name_t;
 
-   /* slaveok is not supported as an option, exhaust is tested separately */
+   /* secondaryOk is not supported as an option, exhaust is tested separately */
    flag_and_name_t flags_and_frags[] = {
       {MONGOC_QUERY_TAILABLE_CURSOR, "'tailable': true"},
       {MONGOC_QUERY_OPLOG_REPLAY, "'oplogReplay': true"},
@@ -809,9 +810,10 @@ test_exhaust (void)
    const bson_t *doc;
    bson_error_t error;
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   server = mock_server_with_auto_hello (WIRE_VERSION_FIND_CMD);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
    cursor = mongoc_collection_find (
       collection, MONGOC_QUERY_EXHAUST, 0, 0, 0, tmp_bson (NULL), NULL, NULL);
@@ -824,7 +826,7 @@ test_exhaust (void)
     */
    request = mock_server_receives_request (server);
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_SLAVE_OK | MONGOC_QUERY_EXHAUST,
+                                MONGOC_QUERY_SECONDARY_OK | MONGOC_QUERY_EXHAUST,
                                 0,
                                 0,
                                 "db.collection",
@@ -858,9 +860,10 @@ test_getmore_batch_size (void)
    char *batch_size_json;
    bson_error_t error;
 
-   server = mock_server_with_autoismaster (4);
+   server = mock_server_with_auto_hello (4);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    for (i = 0; i < sizeof (batch_sizes) / sizeof (uint32_t); i++) {
@@ -885,7 +888,7 @@ test_getmore_batch_size (void)
       request = mock_server_receives_command (
          server,
          "db",
-         MONGOC_QUERY_SLAVE_OK,
+         MONGOC_QUERY_SECONDARY_OK,
          "{'find': 'collection', 'filter': {}, 'batchSize': %s}",
          batch_size_json);
 
@@ -928,9 +931,10 @@ test_getmore_invalid_reply (void *ctx)
       return;
    }
 
-   server = mock_server_with_autoismaster (4);
+   server = mock_server_with_auto_hello (4);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    cursor = mongoc_collection_find (
@@ -940,7 +944,7 @@ test_getmore_invalid_reply (void *ctx)
    request =
       mock_server_receives_command (server,
                                     "db",
-                                    MONGOC_QUERY_SLAVE_OK,
+                                    MONGOC_QUERY_SECONDARY_OK,
                                     "{'find': 'collection', 'filter': {}}");
 
    mock_server_replies_simple (request,
@@ -959,7 +963,7 @@ test_getmore_invalid_reply (void *ctx)
    request = mock_server_receives_command (
       server,
       "db",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'getMore': {'$numberLong': '123'}, 'collection': 'collection'}");
 
    /* missing "cursor" */
@@ -1005,9 +1009,10 @@ test_getmore_await (void)
    size_t i;
    char *max_time_json;
 
-   server = mock_server_with_autoismaster (4);
+   server = mock_server_with_auto_hello (4);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    for (i = 3; i < sizeof (await_tests) / sizeof (await_test_t); i++) {
@@ -1026,11 +1031,11 @@ test_getmore_await (void)
       mongoc_cursor_set_max_await_time_ms (cursor, 123);
       future = future_cursor_next (cursor, &doc);
 
-      /* only the slave ok bit is still in the query header */
+      /* only the secondaryOk bit is still in the query header */
       request = mock_server_receives_command (
          server,
          "db",
-         MONGOC_QUERY_SLAVE_OK,
+         MONGOC_QUERY_SECONDARY_OK,
          "{'find': 'collection',"
          " 'maxTimeMS': {'$exists': false},"
          " 'maxAwaitTimeMS': {'$exists': false}}");
@@ -1058,11 +1063,11 @@ test_getmore_await (void)
          max_time_json = "{'$exists': false}";
       }
 
-      /* only the slave ok bit is still in the query header */
+      /* only the secondaryOk bit is still in the query header */
       request =
          mock_server_receives_command (server,
                                        "db",
-                                       MONGOC_QUERY_SLAVE_OK,
+                                       MONGOC_QUERY_SECONDARY_OK,
                                        "{'getMore': {'$numberLong': '1'},"
                                        " 'collection': 'collection',"
                                        " 'maxAwaitTimeMS': {'$exists': false},"
@@ -1112,10 +1117,10 @@ _test_tailable_timeout (bool pooled)
    capture_logs (true);
 
    if (pooled) {
-      pool = test_framework_client_pool_new ();
+      pool = test_framework_new_default_client_pool ();
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = test_framework_client_new ();
+      client = test_framework_new_default_client ();
    }
 
    database = mongoc_client_get_database (client, "test");
