@@ -8,6 +8,7 @@
 #include "mock_server/future.h"
 #include "mock_server/future-functions.h"
 #include "json-test-operations.h"
+#include "unified/runner.h"
 
 
 const char *
@@ -105,7 +106,10 @@ test_all_spec_tests (TestSuite *suite)
 {
    char resolved[PATH_MAX];
 
-   test_framework_resolve_path (JSON_DIR "/command_monitoring", resolved);
+   run_unified_tests (suite, JSON_DIR "/command_monitoring/unified");
+
+   test_framework_resolve_path (JSON_DIR "/command_monitoring/legacy",
+                                resolved);
    install_json_test_suite (suite, resolved, &test_command_monitoring_cb);
 }
 
@@ -130,17 +134,18 @@ test_get_error (void)
    request_t *request;
    bson_error_t error = {0};
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_MIN);
+   server = mock_server_with_auto_hello (WIRE_VERSION_MIN);
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    callbacks = mongoc_apm_callbacks_new ();
    mongoc_apm_set_command_failed_cb (callbacks, test_get_error_failed_cb);
    mongoc_client_set_apm_callbacks (client, callbacks, (void *) &error);
    future = future_client_command_simple (
       client, "db", tmp_bson ("{'foo': 1}"), NULL, NULL, NULL);
    request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK, "{'foo': 1}");
+      server, "db", MONGOC_QUERY_SECONDARY_OK, "{'foo': 1}");
    mock_server_replies_simple (request,
                                "{'ok': 0, 'errmsg': 'foo', 'code': 42}");
    ASSERT (!future_get_bool (future));
@@ -230,7 +235,7 @@ test_change_callbacks (void)
    inc_callbacks = increment_callbacks ();
    dec_callbacks = decrement_callbacks ();
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_apm_callbacks (client, inc_callbacks, &incremented);
 
    collection = get_test_collection (client, "test_change_callbacks");
@@ -281,7 +286,7 @@ test_reset_callbacks (void)
    inc_callbacks = increment_callbacks ();
    dec_callbacks = decrement_callbacks ();
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = get_test_collection (client, "test_reset_apm_callbacks");
 
    /* insert 200 docs so we have a couple batches */
@@ -353,7 +358,7 @@ _test_set_callbacks (bool pooled, bool try_pop)
    mongoc_apm_set_command_started_cb (callbacks, test_set_callbacks_cb);
 
    if (pooled) {
-      pool = test_framework_client_pool_new ();
+      pool = test_framework_new_default_client_pool ();
       ASSERT (mongoc_client_pool_set_apm_callbacks (
          pool, callbacks, (void *) &n_calls));
       if (try_pop) {
@@ -362,7 +367,7 @@ _test_set_callbacks (bool pooled, bool try_pop)
          client = mongoc_client_pool_pop (pool);
       }
    } else {
-      client = test_framework_client_new ();
+      client = test_framework_new_default_client ();
       ASSERT (mongoc_client_set_apm_callbacks (
          client, callbacks, (void *) &n_calls));
    }
@@ -537,12 +542,12 @@ _test_bulk_operation_id (bool pooled, bool use_bulk_operation_new)
    mongoc_apm_set_command_failed_cb (callbacks, test_op_id_failed_cb);
 
    if (pooled) {
-      pool = test_framework_client_pool_new ();
+      pool = test_framework_new_default_client_pool ();
       ASSERT (mongoc_client_pool_set_apm_callbacks (
          pool, callbacks, (void *) &test));
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = test_framework_client_new ();
+      client = test_framework_new_default_client ();
       ASSERT (
          mongoc_client_set_apm_callbacks (client, callbacks, (void *) &test));
    }
@@ -655,7 +660,7 @@ _test_query_operation_id (bool pooled)
 
    op_id_test_init (&test);
 
-   server = mock_server_with_autoismaster (4);
+   server = mock_server_with_auto_hello (4);
    mock_server_run (server);
 
    callbacks = mongoc_apm_callbacks_new ();
@@ -664,12 +669,14 @@ _test_query_operation_id (bool pooled)
    mongoc_apm_set_command_failed_cb (callbacks, test_op_id_failed_cb);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (mock_server_get_uri (server));
+      pool = test_framework_client_pool_new_from_uri (
+         mock_server_get_uri (server), NULL);
       ASSERT (mongoc_client_pool_set_apm_callbacks (
          pool, callbacks, (void *) &test));
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+      client = test_framework_client_new_from_uri (mock_server_get_uri (server),
+                                                   NULL);
       ASSERT (
          mongoc_client_set_apm_callbacks (client, callbacks, (void *) &test));
    }
@@ -681,7 +688,7 @@ _test_query_operation_id (bool pooled)
    future = future_cursor_next (cursor, &doc);
    request = mock_server_receives_request (server);
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_SLAVE_OK,
+                                MONGOC_QUERY_SECONDARY_OK,
                                 123 /* cursor id */,
                                 1,
                                 "db.collection",
@@ -863,11 +870,11 @@ test_client_cmd (void)
    const bson_t *reply;
 
    cmd_test_init (&test);
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    set_cmd_test_callbacks (client, (void *) &test);
    cursor = mongoc_client_command (client,
                                    "admin",
-                                   MONGOC_QUERY_SLAVE_OK,
+                                   MONGOC_QUERY_SECONDARY_OK,
                                    0,
                                    0,
                                    0,
@@ -889,7 +896,7 @@ test_client_cmd (void)
    cmd_test_init (&test);
    cursor = mongoc_client_command (client,
                                    "admin",
-                                   MONGOC_QUERY_SLAVE_OK,
+                                   MONGOC_QUERY_SECONDARY_OK,
                                    0,
                                    0,
                                    0,
@@ -920,7 +927,7 @@ test_client_cmd_simple (void)
    bson_error_t error;
 
    cmd_test_init (&test);
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    set_cmd_test_callbacks (client, (void *) &test);
    r = mongoc_client_command_simple (
       client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
@@ -968,7 +975,7 @@ test_client_cmd_op_ids (void)
    mongoc_apm_set_command_succeeded_cb (callbacks, test_op_id_succeeded_cb);
    mongoc_apm_set_command_failed_cb (callbacks, test_op_id_failed_cb);
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_apm_callbacks (client, callbacks, (void *) &test);
 
    r = mongoc_client_command_simple (
@@ -1016,7 +1023,7 @@ test_killcursors_deprecated (void)
    bson_error_t error;
 
    cmd_test_init (&test);
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
 
    /* connect */
    r = mongoc_client_command_simple (
@@ -1090,14 +1097,15 @@ test_command_failed_reply_mock (void)
     */
    cmd_failed_reply_test_init (&test);
 
-   server = mock_server_with_autoismaster (4);
+   server = mock_server_with_auto_hello (4);
    mock_server_run (server);
 
    callbacks = mongoc_apm_callbacks_new ();
    mongoc_apm_set_command_failed_cb (callbacks,
                                      command_failed_reply_command_failed_cb);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    ASSERT (mongoc_client_set_apm_callbacks (client, callbacks, (void *) &test));
 
    collection = mongoc_client_get_collection (client, "db", "collection");
@@ -1147,14 +1155,15 @@ test_command_failed_reply_hangup (void)
     * error (i.e. the server hangs up) */
    cmd_failed_reply_test_init (&test);
 
-   server = mock_server_with_autoismaster (4);
+   server = mock_server_with_auto_hello (4);
    mock_server_run (server);
 
    callbacks = mongoc_apm_callbacks_new ();
    mongoc_apm_set_command_failed_cb (callbacks,
                                      command_failed_reply_command_failed_cb);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    ASSERT (mongoc_client_set_apm_callbacks (client, callbacks, (void *) &test));
 
    collection = mongoc_client_get_collection (client, "db", "collection");
