@@ -27,7 +27,7 @@
    do {                                                             \
       bson_error_t _err;                                            \
       bool _ret;                                                    \
-      client = test_framework_client_new ();                        \
+      client = test_framework_new_default_client ();                \
       coll = mongoc_client_get_collection (client, "test", "test"); \
       /* populate to ensure db and coll exist. */                   \
       _ret = mongoc_collection_insert_one (                         \
@@ -44,6 +44,9 @@
 
 typedef mongoc_cursor_t *(*make_cursor_fn) (mongoc_collection_t *);
 
+static mongoc_cursor_t *
+_make_cmd_deprecated_cursor (mongoc_collection_t *coll);
+
 static mongoc_cursor_t *
 _make_array_cursor (mongoc_collection_t *coll);
 
@@ -150,8 +153,8 @@ _test_common_clone_w_concerns (void *ctx)
    mongoc_read_concern_destroy (cursor->read_concern);
    cursor->read_concern = read_concern;
    /* don't call mongoc_cursor_next (), since the test may run against a version
-   * of MongoDB that doesn't support read/write concerns, and we are only
-   * interested in testing if the clone process works. */
+    * of MongoDB that doesn't support read/write concerns, and we are only
+    * interested in testing if the clone process works. */
    cloned = mongoc_cursor_clone (cursor);
    /* test cloned read_concern. */
    ASSERT (!mongoc_read_concern_is_default (cloned->read_concern));
@@ -159,7 +162,8 @@ _test_common_clone_w_concerns (void *ctx)
                   MONGOC_READ_CONCERN_LEVEL_LOCAL);
    /* test cloned write_concern. */
    ASSERT (mongoc_write_concern_get_wmajority (cloned->write_concern));
-   ASSERT (mongoc_write_concern_get_wtimeout_int64 (cloned->write_concern) == 1000);
+   ASSERT (mongoc_write_concern_get_wtimeout_int64 (cloned->write_concern) ==
+           1000);
    ASSERT (mongoc_write_concern_get_w (cloned->write_concern) ==
            MONGOC_WRITE_CONCERN_W_MAJORITY);
    /* check generated bson in cloned cursor. */
@@ -232,8 +236,9 @@ _test_common_server_hint_command_started (
    const char *cmd = mongoc_apm_command_started_get_command_name (event);
    test_common_server_hint_ctx_t *ctx;
    /* only check command associated with cursor priming. */
-   if (strcmp (cmd, "find") == 0 || strcmp (cmd, "ping") == 0 ||
-       strcmp (cmd, "listDatabases") == 0) {
+   if (strcmp (cmd, "find") == 0 ||
+       strcasecmp (cmd, HANDSHAKE_CMD_LEGACY_HELLO) == 0 ||
+       strcmp (cmd, "hello") == 0 || strcmp (cmd, "listDatabases") == 0) {
       ctx = (test_common_server_hint_ctx_t *)
          mongoc_apm_command_started_get_context (event);
       ASSERT_CMPSTR (host->host_and_port, ctx->expected_host_and_port);
@@ -305,8 +310,9 @@ _test_common_opts (void *ctx)
    BSON_ASSERT (mongoc_cursor_set_hint (cursor, sd->id));
    ASSERT_CMPINT (mongoc_cursor_get_hint (cursor), ==, sd->id);
 
-   /* listDatabases prohibits limit and batchSize */
-   if ((make_cursor_fn) ctx != _make_array_cursor) {
+   /* listDatabases and hello prohibits limit and batchSize */
+   if ((make_cursor_fn) ctx != _make_array_cursor &&
+       (make_cursor_fn) ctx != _make_cmd_deprecated_cursor) {
       mongoc_cursor_set_batch_size (cursor, 1);
       ASSERT_CMPINT (mongoc_cursor_get_batch_size (cursor), ==, 1);
       BSON_ASSERT (mongoc_cursor_set_limit (cursor, 2));
@@ -316,10 +322,11 @@ _test_common_opts (void *ctx)
    mongoc_cursor_set_max_await_time_ms (cursor, 3);
    ASSERT_CMPINT (mongoc_cursor_get_max_await_time_ms (cursor), ==, 3);
    /* prime the cursor. */
-   BSON_ASSERT (mongoc_cursor_next (cursor, &doc));
+   ASSERT_OR_PRINT (mongoc_cursor_next (cursor, &doc), cursor->error);
    /* options should be unchanged. */
    ASSERT_CMPINT (mongoc_cursor_get_hint (cursor), ==, sd->id);
-   if ((make_cursor_fn) ctx != _make_array_cursor) {
+   if ((make_cursor_fn) ctx != _make_array_cursor &&
+       (make_cursor_fn) ctx != _make_cmd_deprecated_cursor) {
       ASSERT_CMPINT (mongoc_cursor_get_batch_size (cursor), ==, 1);
       ASSERT_CMPINT ((int) mongoc_cursor_get_limit (cursor), ==, 2);
       /* limit cannot be set again. */
@@ -423,21 +430,22 @@ static mongoc_cursor_t *
 _make_cmd_cursor_from_agg (mongoc_collection_t *coll)
 {
    return mongoc_collection_aggregate (
-      coll, MONGOC_QUERY_SLAVE_OK, tmp_bson ("{}"), NULL, NULL);
+      coll, MONGOC_QUERY_SECONDARY_OK, tmp_bson ("{}"), NULL, NULL);
 }
 
 
 static mongoc_cursor_t *
 _make_cmd_deprecated_cursor (mongoc_collection_t *coll)
 {
-   return mongoc_collection_command (coll,
-                                     MONGOC_QUERY_SLAVE_OK,
-                                     0,
-                                     0,
-                                     0,
-                                     tmp_bson ("{'ping': 1}"),
-                                     NULL,
-                                     NULL);
+   return mongoc_collection_command (
+      coll,
+      MONGOC_QUERY_SECONDARY_OK,
+      0,
+      0,
+      0,
+      tmp_bson ("{'" HANDSHAKE_CMD_LEGACY_HELLO "': 1}"),
+      NULL,
+      NULL);
 }
 
 
@@ -533,7 +541,7 @@ test_limit (void)
    const bson_t *doc = NULL;
    bool r;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = get_test_collection (client, "test_limit");
    bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    b = tmp_bson ("{}");
@@ -663,7 +671,7 @@ test_kill_cursor_live (void)
 
    callbacks = mongoc_apm_callbacks_new ();
    mongoc_apm_set_command_succeeded_cb (callbacks, killcursors_succeeded);
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_apm_callbacks (client, callbacks, &ctx);
    collection = get_test_collection (client, "test");
    b = tmp_bson ("{}");
@@ -736,8 +744,7 @@ _test_kill_cursors (bool pooled, bool use_killcursors_cmd)
    const char *ns_out;
    int64_t cursor_id_out;
 
-   rs =
-      mock_rs_with_autoismaster (use_killcursors_cmd ? 4 : 3, /* wire version */
+   rs = mock_rs_with_auto_hello (use_killcursors_cmd ? 4 : 3, /* wire version */
                                  true,                        /* has primary */
                                  5,  /* number of secondaries */
                                  0); /* number of arbiters */
@@ -745,10 +752,11 @@ _test_kill_cursors (bool pooled, bool use_killcursors_cmd)
    mock_rs_run (rs);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (mock_rs_get_uri (rs));
+      pool =
+         test_framework_client_pool_new_from_uri (mock_rs_get_uri (rs), NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (mock_rs_get_uri (rs));
+      client = test_framework_client_new_from_uri (mock_rs_get_uri (rs), NULL);
    }
 
    collection = mongoc_client_get_collection (client, "db", "collection");
@@ -762,7 +770,7 @@ _test_kill_cursors (bool pooled, bool use_killcursors_cmd)
 
    /* reply as appropriate to OP_QUERY or find command */
    mock_rs_replies_to_find (request,
-                            MONGOC_QUERY_SLAVE_OK,
+                            MONGOC_QUERY_SECONDARY_OK,
                             123,
                             1,
                             "db.collection",
@@ -783,7 +791,7 @@ _test_kill_cursors (bool pooled, bool use_killcursors_cmd)
 
    if (use_killcursors_cmd) {
       kill_cursors =
-         mock_rs_receives_command (rs, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+         mock_rs_receives_command (rs, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
 
       /* mock server framework can't test "cursors" array, CDRIVER-994 */
       ASSERT (BCON_EXTRACT ((bson_t *) request_get_doc (kill_cursors, 0),
@@ -868,12 +876,12 @@ _test_client_kill_cursor (bool has_primary, bool wire_version_4)
    future_t *future;
    request_t *request;
 
-   rs = mock_rs_with_autoismaster (wire_version_4 ? 4 : 3,
-                                   has_primary, /* maybe a primary*/
-                                   1,           /* definitely a secondary */
-                                   0);          /* no arbiter */
+   rs = mock_rs_with_auto_hello (wire_version_4 ? 4 : 3,
+                                 has_primary, /* maybe a primary*/
+                                 1,           /* definitely a secondary */
+                                 0);          /* no arbiter */
    mock_rs_run (rs);
-   client = mongoc_client_new_from_uri (mock_rs_get_uri (rs));
+   client = test_framework_client_new_from_uri (mock_rs_get_uri (rs), NULL);
    read_prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
 
    /* make client open a connection - it won't open one to kill a cursor */
@@ -881,7 +889,7 @@ _test_client_kill_cursor (bool has_primary, bool wire_version_4)
       client, "admin", tmp_bson ("{'foo': 1}"), read_prefs, NULL, &error);
 
    request =
-      mock_rs_receives_command (rs, "admin", MONGOC_QUERY_SLAVE_OK, NULL);
+      mock_rs_receives_command (rs, "admin", MONGOC_QUERY_SECONDARY_OK, NULL);
 
    mock_rs_replies_simple (request, "{'ok': 1}");
    ASSERT_OR_PRINT (future_get_bool (future), error);
@@ -972,7 +980,7 @@ _test_cursor_new_from_command (const char *cmd_json,
    bson_t reply;
    mongoc_cursor_t *cmd_cursor;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = mongoc_client_get_collection (client, "test", collection_name);
    mongoc_collection_delete_many (
       collection, tmp_bson ("{}"), NULL, NULL, NULL);
@@ -1011,7 +1019,7 @@ test_cursor_empty_collection (void)
    const bson_t *doc;
    mongoc_cursor_t *cursor;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = mongoc_client_get_collection (
       client, "test", "test_cursor_empty_collection");
    mongoc_collection_delete_many (
@@ -1081,7 +1089,7 @@ test_cursor_new_invalid (void)
    bson_t b = BSON_INITIALIZER;
    const bson_t *error_doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    cursor = mongoc_cursor_new_from_command_reply_with_opts (client, &b, NULL);
    ASSERT (cursor);
    ASSERT (mongoc_cursor_error (cursor, &error));
@@ -1109,10 +1117,11 @@ test_cursor_new_tailable_await (void)
    future_t *future;
    request_t *request;
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   server = mock_server_with_auto_hello (WIRE_VERSION_FIND_CMD);
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    cursor = mongoc_cursor_new_from_command_reply_with_opts (
       client,
       bson_copy (tmp_bson ("{'ok': 1,"
@@ -1135,13 +1144,13 @@ test_cursor_new_tailable_await (void)
    future = future_cursor_next (cursor, &doc);
    request = mock_server_receives_command (server,
                                            "db",
-                                           MONGOC_QUERY_SLAVE_OK,
+                                           MONGOC_QUERY_SECONDARY_OK,
                                            "{'getMore': {'$numberLong': '123'},"
                                            " 'collection': 'collection',"
                                            " 'maxTimeMS': 100"
                                            "}");
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_SLAVE_OK,
+                                MONGOC_QUERY_SECONDARY_OK,
                                 0 /* cursor id */,
                                 1 /* number returned */,
                                 "db.collection",
@@ -1172,10 +1181,11 @@ test_cursor_int64_t_maxtimems (void)
    bson_t *max_await_time_ms;
    uint64_t ms_int64 = UINT32_MAX + (uint64_t) 1;
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   server = mock_server_with_auto_hello (WIRE_VERSION_FIND_CMD);
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
 
    max_await_time_ms = tmp_bson (NULL);
    bson_append_bool (max_await_time_ms, "tailable", 8, true);
@@ -1202,14 +1212,14 @@ test_cursor_int64_t_maxtimems (void)
    request = mock_server_receives_command (
       server,
       "db",
-      MONGOC_QUERY_SLAVE_OK,
+      MONGOC_QUERY_SECONDARY_OK,
       "{'getMore': {'$numberLong': '123'},"
       " 'collection': 'collection',"
       " 'maxTimeMS': {'$numberLong': '%" PRIu64 "'}"
       "}",
       ms_int64);
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_SLAVE_OK,
+                                MONGOC_QUERY_SECONDARY_OK,
                                 0 /* cursor id */,
                                 1 /* number returned */,
                                 "db.collection",
@@ -1236,10 +1246,11 @@ test_cursor_new_ignores_fields (void)
    const bson_t *doc;
    bson_error_t error;
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   server = mock_server_with_auto_hello (WIRE_VERSION_FIND_CMD);
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    cursor = mongoc_cursor_new_from_command_reply_with_opts (
       client,
       bson_copy (tmp_bson ("{'ok': 1,"
@@ -1274,7 +1285,7 @@ test_cursor_new_invalid_filter (void)
    bson_error_t error;
    const bson_t *error_doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = mongoc_client_get_collection (client, "test", "test");
 
    cursor = mongoc_collection_find_with_opts (
@@ -1305,7 +1316,7 @@ test_cursor_new_invalid_opts (void)
    bson_error_t error;
    const bson_t *error_doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = mongoc_client_get_collection (client, "test", "test");
 
    cursor = mongoc_collection_find_with_opts (
@@ -1362,7 +1373,7 @@ test_cursor_new_static (void)
       &bson_static, bson_get_data (bson_alloced), bson_alloced->len));
 
    /* test heap-allocated bson */
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    bson_copied = bson_copy (bson_alloced);
    cursor = mongoc_cursor_new_from_command_reply_with_opts (
       client, bson_copied, NULL);
@@ -1389,7 +1400,7 @@ test_cursor_hint_errors (void)
    mongoc_collection_t *collection;
    mongoc_cursor_t *cursor;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    collection = mongoc_client_get_collection (client, "db", "collection");
    cursor = mongoc_collection_find (
       collection, MONGOC_QUERY_NONE, 0, 0, 0, tmp_bson ("{}"), NULL, NULL);
@@ -1456,14 +1467,15 @@ _test_cursor_hint (bool pooled, bool use_primary)
    request_t *request;
 
    /* wire version WIRE_VERSION_MIN, primary, two secondaries, no arbiters */
-   rs = mock_rs_with_autoismaster (WIRE_VERSION_MIN, true, 2, 0);
+   rs = mock_rs_with_auto_hello (WIRE_VERSION_MIN, true, 2, 0);
    mock_rs_run (rs);
 
    if (pooled) {
-      pool = mongoc_client_pool_new (mock_rs_get_uri (rs));
+      pool =
+         test_framework_client_pool_new_from_uri (mock_rs_get_uri (rs), NULL);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (mock_rs_get_uri (rs));
+      client = test_framework_client_new_from_uri (mock_rs_get_uri (rs), NULL);
    }
 
    collection = mongoc_client_get_collection (client, "test", "test");
@@ -1477,7 +1489,7 @@ _test_cursor_hint (bool pooled, bool use_primary)
       expected_flags = MONGOC_QUERY_NONE;
    } else {
       server_id = server_id_for_read_mode (client, MONGOC_READ_SECONDARY);
-      expected_flags = MONGOC_QUERY_SLAVE_OK;
+      expected_flags = MONGOC_QUERY_SECONDARY_OK;
    }
 
    ASSERT (mongoc_cursor_set_hint (cursor, server_id));
@@ -1547,13 +1559,13 @@ mongoc_read_mode_t modes[] = {
 
 mongoc_query_flags_t expected_flag[] = {
    MONGOC_QUERY_NONE,
-   MONGOC_QUERY_SLAVE_OK,
-   MONGOC_QUERY_SLAVE_OK,
-   MONGOC_QUERY_SLAVE_OK,
-   MONGOC_QUERY_SLAVE_OK,
+   MONGOC_QUERY_SECONDARY_OK,
+   MONGOC_QUERY_SECONDARY_OK,
+   MONGOC_QUERY_SECONDARY_OK,
+   MONGOC_QUERY_SECONDARY_OK,
 };
 
-/* test that mongoc_cursor_set_hint sets slaveok for mongos only if read pref
+/* test that mongoc_cursor_set_hint sets secondaryOk for mongos only if read pref
  * is secondaryPreferred. */
 static void
 test_cursor_hint_mongos (void)
@@ -1570,7 +1582,8 @@ test_cursor_hint_mongos (void)
 
    server = mock_mongos_new (WIRE_VERSION_MIN);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    for (i = 0; i < sizeof (modes) / sizeof (mongoc_read_mode_t); i++) {
@@ -1616,7 +1629,8 @@ test_cursor_hint_mongos_cmd (void)
 
    server = mock_mongos_new (WIRE_VERSION_FIND_CMD);
    mock_server_run (server);
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "test", "test");
 
    for (i = 0; i < sizeof (modes) / sizeof (mongoc_read_mode_t); i++) {
@@ -1654,7 +1668,7 @@ test_cursor_hint_mongos_cmd (void)
 }
 
 
-/* Tests CDRIVER-562: after calling ismaster to handshake a new connection we
+/* Tests CDRIVER-562: after calling hello to handshake a new connection we
  * must update topology description with the server response. If not, this test
  * fails under auth with "auth failed" because we use the wrong auth protocol.
  */
@@ -1670,10 +1684,10 @@ _test_cursor_hint_no_warmup (bool pooled)
    bson_error_t error;
 
    if (pooled) {
-      pool = test_framework_client_pool_new ();
+      pool = test_framework_new_default_client_pool ();
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = test_framework_client_new ();
+      client = test_framework_new_default_client ();
    }
 
    collection = get_test_collection (client, "test_cursor_hint_no_warmup");
@@ -1723,7 +1737,7 @@ test_tailable_alive (void)
    mongoc_cursor_t *cursor;
    const bson_t *doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    database = mongoc_client_get_database (client, "test");
    collection_name = gen_collection_name ("test");
 
@@ -1858,7 +1872,7 @@ _test_cursor_n_return_find_cmd (mongoc_cursor_t *cursor,
 
    future = future_cursor_next (cursor, &doc);
    request =
-      mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+      mock_server_receives_command (server, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
 
    ASSERT (match_bson (request_get_doc (request, 0), &find_cmd, true));
 
@@ -1880,7 +1894,7 @@ _test_cursor_n_return_find_cmd (mongoc_cursor_t *cursor,
       /* expect getMore command, send reply_length[reply_no] docs to client */
       future = future_cursor_next (cursor, &doc);
       request = mock_server_receives_command (
-         server, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+         server, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
 
       bson_reinit (&getmore_cmd);
       BSON_APPEND_INT64 (&getmore_cmd, "getMore", 123);
@@ -1893,7 +1907,7 @@ _test_cursor_n_return_find_cmd (mongoc_cursor_t *cursor,
             &getmore_cmd, "batchSize", tmp_bson ("{'$exists': false}"));
       }
 
-      ASSERT (match_bson (request_get_doc (request, 0), &getmore_cmd, true));
+      assert_match_bson (request_get_doc (request, 0), &getmore_cmd, true);
 
       reply = bson_string_new (NULL);
       cursor_finished = (reply_no == 2);
@@ -1974,11 +1988,12 @@ _test_cursor_n_return (bool find_with_opts)
    bson_t opts = BSON_INITIALIZER;
    mongoc_cursor_t *cursor;
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   server = mock_server_with_auto_hello (WIRE_VERSION_FIND_CMD);
 
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "coll");
 
    for (i = 0; i < sizeof (tests) / sizeof (cursor_n_return_test); i++) {
@@ -2054,7 +2069,7 @@ test_empty_final_batch_live (void)
    int i;
    bool r;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_error_api (client, 2);
    collection = get_test_collection (client, "test_empty_final_batch_live");
 
@@ -2098,10 +2113,11 @@ test_empty_final_batch (void)
    request_t *request;
    bson_error_t error;
 
-   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   server = mock_server_with_auto_hello (WIRE_VERSION_FIND_CMD);
    mock_server_run (server);
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    collection = mongoc_client_get_collection (client, "db", "coll");
    cursor = mongoc_collection_find_with_opts (
       collection,
@@ -2114,10 +2130,10 @@ test_empty_final_batch (void)
     */
    future = future_cursor_next (cursor, &doc);
    request =
-      mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+      mock_server_receives_command (server, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
 
    mock_server_replies_to_find (
-      request, MONGOC_QUERY_SLAVE_OK, 1234, 0, "db.coll", "{}", true);
+      request, MONGOC_QUERY_SECONDARY_OK, 1234, 0, "db.coll", "{}", true);
 
    ASSERT (future_get_bool (future));
    future_destroy (future);
@@ -2128,10 +2144,10 @@ test_empty_final_batch (void)
     */
    future = future_cursor_next (cursor, &doc);
    request =
-      mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+      mock_server_receives_command (server, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
 
    mock_server_replies_to_find (
-      request, MONGOC_QUERY_SLAVE_OK, 1234, 0, "db.coll", "" /* empty */, true);
+      request, MONGOC_QUERY_SECONDARY_OK, 1234, 0, "db.coll", "" /* empty */, true);
 
    ASSERT (!future_get_bool (future));
    ASSERT_OR_PRINT (!mongoc_cursor_error (cursor, &error), error);
@@ -2143,7 +2159,7 @@ test_empty_final_batch (void)
     */
    future = future_cursor_next (cursor, &doc);
    request =
-      mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK, NULL);
+      mock_server_receives_command (server, "db", MONGOC_QUERY_SECONDARY_OK, NULL);
 
    ASSERT_CMPINT64 (
       bson_lookup_int64 (request_get_doc (request, 0), "batchSize"),
@@ -2151,7 +2167,7 @@ test_empty_final_batch (void)
       (int64_t) 1);
 
    mock_server_replies_to_find (request,
-                                MONGOC_QUERY_SLAVE_OK,
+                                MONGOC_QUERY_SECONDARY_OK,
                                 0 /* cursor id */,
                                 0,
                                 "db.coll",
@@ -2180,7 +2196,7 @@ test_error_document_query (void)
    const bson_t *doc;
    const bson_t *error_doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_error_api (client, 2);
    collection = get_test_collection (client, "test_error_document_query");
    cursor = mongoc_collection_find_with_opts (
@@ -2208,7 +2224,7 @@ test_error_document_command (void)
    const bson_t *doc;
    const bson_t *error_doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_error_api (client, 2);
    cursor = mongoc_client_command (client,
                                    "test",
@@ -2245,7 +2261,7 @@ test_error_document_getmore (void)
    const bson_t *doc;
    const bson_t *error_doc;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_error_api (client, 2);
    collection = get_test_collection (client, "test_error_document_getmore");
    mongoc_collection_drop (collection, NULL);
@@ -2298,7 +2314,7 @@ test_find_error_is_alive (void)
    mongoc_cursor_t *cursor;
    bson_error_t err;
    const bson_t *bson;
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    coll = mongoc_client_get_collection (client, "test", "test");
    cursor =
       mongoc_collection_find (coll,
