@@ -73,7 +73,7 @@ test_sasl_canonicalize_hostname (void *ctx)
    char real_name[BSON_HOST_NAME_MAX + 1] = {'\0'};
    bson_error_t error;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    ss = mongoc_cluster_stream_for_reads (
       &client->cluster, NULL, NULL, NULL, &error);
    ASSERT_OR_PRINT (ss, error);
