@@ -61,7 +61,7 @@ test_mongoc_handshake_appname_in_uri (void)
    capture_logs (true);
    uri_str = bson_strdup_printf ("mongodb://a/?" MONGOC_URI_APPNAME "=%s",
                                  long_string);
-   ASSERT (!mongoc_client_new (uri_str));
+   ASSERT (!test_framework_client_new (uri_str, NULL));
    ASSERT_CAPTURED_LOG ("_mongoc_topology_scanner_set_appname",
                         MONGOC_LOG_LEVEL_WARNING,
                         "Unsupported value");
@@ -92,7 +92,7 @@ test_mongoc_handshake_appname_frozen_single (void)
    mongoc_client_t *client;
    const char *good_uri = "mongodb://host/?" MONGOC_URI_APPNAME "=mongodump";
 
-   client = mongoc_client_new (good_uri);
+   client = test_framework_client_new (good_uri, NULL);
 
    /* Shouldn't be able to set appname again */
    capture_logs (true);
@@ -114,7 +114,7 @@ test_mongoc_handshake_appname_frozen_pooled (void)
 
    uri = mongoc_uri_new (good_uri);
 
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
    capture_logs (true);
    ASSERT (!mongoc_client_pool_set_appname (pool, "test"));
    ASSERT_CAPTURED_LOG ("_mongoc_topology_scanner_set_appname",
@@ -189,16 +189,15 @@ test_mongoc_handshake_data_append_success (void)
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    mongoc_uri_set_option_as_int32 (uri, MONGOC_URI_HEARTBEATFREQUENCYMS, 500);
    mongoc_uri_set_option_as_utf8 (uri, MONGOC_URI_APPNAME, "testapp");
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
 
    /* Force topology scanner to start */
    client = mongoc_client_pool_pop (pool);
 
-   request = mock_server_receives_ismaster (server);
+   request = mock_server_receives_legacy_hello (server, NULL);
    ASSERT (request);
    request_doc = request_get_doc (request, 0);
    ASSERT (request_doc);
-   ASSERT (bson_has_field (request_doc, "isMaster"));
    ASSERT (bson_has_field (request_doc, HANDSHAKE_FIELD));
 
    ASSERT (bson_iter_init_find (&iter, request_doc, HANDSHAKE_FIELD));
@@ -264,7 +263,7 @@ test_mongoc_handshake_data_append_success (void)
       ASSERT (strstr (val, platform) != NULL);
    }
 
-   mock_server_replies_simple (request, "{'ok': 1, 'ismaster': true}");
+   mock_server_replies_simple (request, "{'ok': 1, 'isWritablePrimary': true}");
    request_destroy (request);
 
    /* Cleanup */
@@ -350,7 +349,7 @@ test_mongoc_handshake_data_append_after_cmd (void)
    uri = mongoc_uri_new ("mongodb://127.0.0.1/?" MONGOC_URI_MAXPOOLSIZE "=1");
 
    /* Make sure that after we pop a client we can't set global handshake */
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
 
    client = mongoc_client_pool_pop (pool);
 
@@ -378,7 +377,7 @@ test_mongoc_handshake_too_big (void)
    mongoc_uri_t *uri;
    future_t *future;
    request_t *request;
-   const bson_t *ismaster_doc;
+   const bson_t *hello_doc;
    bson_iter_t iter;
 
    enum { BUFFER_SIZE = HANDSHAKE_MAX_SIZE };
@@ -398,24 +397,23 @@ test_mongoc_handshake_too_big (void)
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    /* avoid rare test timeouts */
    mongoc_uri_set_option_as_int32 (uri, MONGOC_URI_CONNECTTIMEOUTMS, 20000);
-   client = mongoc_client_new_from_uri (uri);
+   client = test_framework_client_new_from_uri (uri, NULL);
 
    ASSERT (mongoc_client_set_appname (client, "my app"));
 
    /* Send a ping, mock server deals with it */
    future = future_client_command_simple (
       client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, NULL);
-   request = mock_server_receives_ismaster (server);
+   request = mock_server_receives_legacy_hello (server, NULL);
 
-   /* Make sure the isMaster request has a handshake field, and it's not huge */
+   /* Make sure the hello request has a handshake field, and it's not huge */
    ASSERT (request);
-   ismaster_doc = request_get_doc (request, 0);
-   ASSERT (ismaster_doc);
-   ASSERT (bson_has_field (ismaster_doc, "isMaster"));
-   ASSERT (bson_has_field (ismaster_doc, HANDSHAKE_FIELD));
+   hello_doc = request_get_doc (request, 0);
+   ASSERT (hello_doc);
+   ASSERT (bson_has_field (hello_doc, HANDSHAKE_FIELD));
 
-   /* isMaster with handshake isn't too big */
-   bson_iter_init_find (&iter, ismaster_doc, HANDSHAKE_FIELD);
+   /* hello with handshake isn't too big */
+   bson_iter_init_find (&iter, hello_doc, HANDSHAKE_FIELD);
    ASSERT (BSON_ITER_HOLDS_DOCUMENT (&iter));
    bson_iter_document (&iter, &len, &dummy);
 
@@ -427,7 +425,7 @@ test_mongoc_handshake_too_big (void)
    request_destroy (request);
 
    request = mock_server_receives_command (
-      server, "admin", MONGOC_QUERY_SLAVE_OK, "{'ping': 1}");
+      server, "admin", MONGOC_QUERY_SECONDARY_OK, "{'ping': 1}");
 
    mock_server_replies_simple (request, "{'ok': 1}");
    ASSERT (future_get_bool (future));
@@ -547,7 +545,7 @@ test_mongoc_handshake_cannot_send (void)
    mongoc_client_t *client;
    mongoc_client_pool_t *pool;
    request_t *request;
-   const char *const server_reply = "{'ok': 1, 'ismaster': true}";
+   const char *const server_reply = "{'ok': 1, 'isWritablePrimary': true}";
    const bson_t *request_doc;
    char big_string[HANDSHAKE_MAX_SIZE];
    mongoc_handshake_t *md;
@@ -568,35 +566,33 @@ test_mongoc_handshake_cannot_send (void)
    mock_server_run (server);
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    mongoc_uri_set_option_as_int32 (uri, MONGOC_URI_HEARTBEATFREQUENCYMS, 500);
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
 
    /* Pop a client to trigger the topology scanner */
    client = mongoc_client_pool_pop (pool);
-   request = mock_server_receives_ismaster (server);
+   request = mock_server_receives_legacy_hello (server, NULL);
 
-   /* Make sure the isMaster request DOESN'T have a handshake field: */
+   /* Make sure the hello request DOESN'T have a handshake field: */
    ASSERT (request);
    request_doc = request_get_doc (request, 0);
    ASSERT (request_doc);
-   ASSERT (bson_has_field (request_doc, "isMaster"));
    ASSERT (!bson_has_field (request_doc, HANDSHAKE_FIELD));
 
    mock_server_replies_simple (request, server_reply);
    request_destroy (request);
 
    /* Cause failure on client side */
-   request = mock_server_receives_ismaster (server);
+   request = mock_server_receives_legacy_hello (server, NULL);
    ASSERT (request);
    mock_server_hangs_up (request);
    request_destroy (request);
 
-   /* Make sure the isMaster request still DOESN'T have a handshake field
+   /* Make sure the hello request still DOESN'T have a handshake field
     * on subsequent heartbeats. */
-   request = mock_server_receives_ismaster (server);
+   request = mock_server_receives_legacy_hello (server, NULL);
    ASSERT (request);
    request_doc = request_get_doc (request, 0);
    ASSERT (request_doc);
-   ASSERT (bson_has_field (request_doc, "isMaster"));
    ASSERT (!bson_has_field (request_doc, HANDSHAKE_FIELD));
 
    mock_server_replies_simple (request, server_reply);
