@@ -476,7 +476,7 @@ test_sdam_monitoring_cb (bson_t *test)
 
    /* parse out the uri and use it to create a client */
    BSON_ASSERT (bson_iter_init_find (&iter, test, "uri"));
-   client = mongoc_client_new (bson_iter_utf8 (&iter, NULL));
+   client = test_framework_client_new (bson_iter_utf8 (&iter, NULL), NULL);
    topology = client->topology;
    context_init (&context);
    client_set_topology_event_callbacks (client, &context);
@@ -501,7 +501,7 @@ test_sdam_monitoring_cb (bson_t *test)
          bson_reinit (&context.events);
          context.n_events = 0;
       }
-      process_sdam_test_ismaster_responses (&phase, client->topology);
+      process_sdam_test_hello_responses (&phase, client->topology);
 
       /* parse out "outcome" and validate */
       BSON_ASSERT (bson_iter_init_find (&phase_field_iter, &phase, "outcome"));
@@ -558,11 +558,11 @@ _test_topology_events (bool pooled)
    context_init (&context);
 
    if (pooled) {
-      pool = test_framework_client_pool_new ();
+      pool = test_framework_new_default_client_pool ();
       pool_set_topology_event_callbacks (pool, &context);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = test_framework_client_new ();
+      client = test_framework_new_default_client ();
       client_set_topology_event_callbacks (client, &context);
    }
 
@@ -622,7 +622,7 @@ test_topology_events_disabled (void)
 
    context_init (&context);
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    client_set_topology_event_callbacks (client, &context);
 
    r = mongoc_client_command_simple (
@@ -698,20 +698,20 @@ _test_heartbeat_events (bool pooled, bool succeeded)
    start = bson_get_monotonic_time ();
 
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       pool_set_heartbeat_event_callbacks (pool, &context);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
       client_set_heartbeat_event_callbacks (client, &context);
    }
 
-   /* trigger "ismaster" handshake */
+   /* trigger "hello" handshake */
    future = future_client_command_simple (
       client, "admin", tmp_bson ("{'foo': 1}"), NULL, NULL, &error);
 
-   /* topology scanner calls ismaster once */
-   request = mock_server_receives_ismaster (server);
+   /* topology scanner calls hello once */
+   request = mock_server_receives_legacy_hello (server, NULL);
 
    if (succeeded) {
       mock_server_replies (
@@ -727,9 +727,9 @@ _test_heartbeat_events (bool pooled, bool succeeded)
       request_destroy (request);
    }
 
-   /* pooled client opens new socket, handshakes it by calling ismaster again */
+   /* pooled client opens new socket, handshakes it by calling hello again */
    if (pooled && succeeded) {
-      request = mock_server_receives_ismaster (server);
+      request = mock_server_receives_legacy_hello (server, NULL);
       mock_server_replies (
          request,
          MONGOC_REPLY_NONE,
@@ -835,15 +835,15 @@ _test_heartbeat_fails_dns (bool pooled)
    uri = mongoc_uri_new (
       "mongodb://doesntexist.foobar/?serverSelectionTimeoutMS=3000");
    if (pooled) {
-      pool = mongoc_client_pool_new (uri);
+      pool = test_framework_client_pool_new_from_uri (uri, NULL);
       pool_set_heartbeat_event_callbacks (pool, &context);
       client = mongoc_client_pool_pop (pool);
    } else {
-      client = mongoc_client_new_from_uri (uri);
+      client = test_framework_client_new_from_uri (uri, NULL);
       client_set_heartbeat_event_callbacks (client, &context);
    }
 
-   /* trigger "ismaster" handshake */
+   /* trigger "hello" handshake */
    r = mongoc_client_command_simple (
       client, "admin", tmp_bson ("{'foo': 1}"), NULL, NULL, &error);
 
@@ -912,7 +912,7 @@ duplicates_topology_changed (const mongoc_apm_topology_changed_t *event)
    counters->num_topology_description_changed_events++;
 }
 
-/* Test that duplicate ismaster responses do not trigger two server
+/* Test that duplicate hello responses do not trigger two server
  * description changed events or topology changed events */
 static void
 test_no_duplicates (void)
@@ -924,12 +924,12 @@ test_no_duplicates (void)
    bson_error_t error;
    future_t *future;
    mongoc_uri_t *uri;
-   char *first_ismaster_response =
-      "{'ok': 1.0, 'ismaster': true, 'minWireVersion': 0, 'maxWireVersion': 9}";
-   char *second_ismaster_response = "{'ok': 1.0, 'ismaster': true, "
-                                    "'minWireVersion': 0, 'maxWireVersion': 9, "
-                                    "'lastWrite': {'lastWriteDate': {'$date': "
-                                    "{'$numberLong': '123'}}, 'opTime': 2}}";
+   char *first_hello_response = "{'ok': 1.0, 'isWritablePrimary': true, "
+                                "'minWireVersion': 0, 'maxWireVersion': 9}";
+   char *second_hello_response = "{'ok': 1.0, 'isWritablePrimary': true, "
+                                 "'minWireVersion': 0, 'maxWireVersion': 9, "
+                                 "'lastWrite': {'lastWriteDate': {'$date': "
+                                 "{'$numberLong': '123'}}, 'opTime': 2}}";
    mongoc_apm_callbacks_t *callbacks;
    duplicates_counter_t duplicates_counter = {0};
    mongoc_server_description_t *sd;
@@ -942,27 +942,27 @@ test_no_duplicates (void)
     * from interfering. */
    uri = mongoc_uri_copy (mock_server_get_uri (server));
    mongoc_uri_set_option_as_int32 (uri, MONGOC_URI_HEARTBEATFREQUENCYMS, 99999);
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
    mongoc_apm_set_server_changed_cb (callbacks, duplicates_server_changed);
    mongoc_apm_set_topology_changed_cb (callbacks, duplicates_topology_changed);
    mongoc_client_pool_set_apm_callbacks (pool, callbacks, &duplicates_counter);
    client = mongoc_client_pool_pop (pool);
 
-   /* Topology scanning thread starts, and sends an ismaster. */
-   request = mock_server_receives_ismaster (server);
-   mock_server_replies_simple (request, first_ismaster_response);
+   /* Topology scanning thread starts, and sends a hello. */
+   request = mock_server_receives_legacy_hello (server, NULL);
+   mock_server_replies_simple (request, first_hello_response);
    request_destroy (request);
 
    /* Perform a ping, which creates a new connection, which performs the
-    * ismaster handshake before sending the ping command. */
+    * hello handshake before sending the ping command. */
    future = future_client_command_simple (client,
                                           "admin",
                                           tmp_bson ("{'ping': 1}"),
                                           NULL /* read prefs */,
                                           NULL /* reply */,
                                           &error);
-   request = mock_server_receives_ismaster (server);
-   mock_server_replies_simple (request, second_ismaster_response);
+   request = mock_server_receives_legacy_hello (server, NULL);
+   mock_server_replies_simple (request, second_hello_response);
    request_destroy (request);
    request = mock_server_receives_msg (
       server, MONGOC_QUERY_NONE, tmp_bson ("{'ping': 1}"));
@@ -973,7 +973,7 @@ test_no_duplicates (void)
    ASSERT_CMPINT (
       duplicates_counter.num_server_description_changed_events, ==, 1);
    /* There should be two topology changed events. One for the initial topology
-    * (where the server is set to Unknown), and one for the first ismaster (but
+    * (where the server is set to Unknown), and one for the first hello (but
     * not the second) */
    ASSERT_CMPINT (
       duplicates_counter.num_topology_description_changed_events, ==, 2);
@@ -983,7 +983,7 @@ test_no_duplicates (void)
     * topology description. It differs in that it has the 'lastWrite' field,
     * which does not have an effect in equality comparison. */
    sd = mongoc_client_get_server_description (client, 1);
-   BSON_ASSERT (bson_has_field (&sd->last_is_master, "lastWrite"));
+   BSON_ASSERT (bson_has_field (&sd->last_hello_response, "lastWrite"));
    mongoc_server_description_destroy (sd);
 
    mongoc_uri_destroy (uri);
