@@ -121,7 +121,7 @@ test_sdam_cb (bson_t *test)
 
    /* parse out the uri and use it to create a client */
    BSON_ASSERT (bson_iter_init_find (&iter, test, "uri"));
-   client = mongoc_client_new (bson_iter_utf8 (&iter, NULL));
+   client = test_framework_client_new (bson_iter_utf8 (&iter, NULL), NULL);
    td = &client->topology->description;
 
    /* for each phase, parse and validate */
@@ -132,7 +132,7 @@ test_sdam_cb (bson_t *test)
    while (bson_iter_next (&phase_iter)) {
       bson_iter_bson (&phase_iter, &phase);
 
-      process_sdam_test_ismaster_responses (&phase, client->topology);
+      process_sdam_test_hello_responses (&phase, client->topology);
 
       /* parse out "outcome" and validate */
       BSON_ASSERT (bson_iter_init_find (&phase_field_iter, &phase, "outcome"));
@@ -219,7 +219,7 @@ test_sdam_cb (bson_t *test)
 /* Initialize a test context to run one SDAM integration test file.
  *
  * Do not use json_test_ctx_init to initialize a context. It sends commands to
- * check for sessions support. That interferes with failpoints set on isMaster.
+ * check for sessions support. That interferes with failpoints set on hello.
  */
 static void
 sdam_json_test_ctx_init (json_test_ctx_t *ctx,
@@ -338,13 +338,13 @@ run_one_integration_test (json_test_config_t *config, bson_t *test)
    db_name = bson_lookup_utf8 (config->scenario, "database_name");
    coll_name = bson_lookup_utf8 (config->scenario, "collection_name");
 
-   /* SDAM integration tests require streamable ismaster support, which is only
+   /* SDAM integration tests require streamable hello support, which is only
     * available for a client pool. */
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
    mongoc_client_pool_set_error_api (pool, MONGOC_ERROR_API_VERSION_2);
    test_framework_set_pool_ssl_opts (pool);
 
-   setup_client = test_framework_client_new ();
+   setup_client = test_framework_new_default_client ();
    /* Disable failpoints that may have been enabled in a previous test run. */
    deactivate_failpoints_on_all_servers (setup_client);
    mongoc_client_command_simple (setup_client,
@@ -361,7 +361,7 @@ run_one_integration_test (json_test_config_t *config, bson_t *test)
    }
 
    /* Listen for events before topology scanning starts. Some tests
-    * check the result of the first ismaster command. But popping a client
+    * check the result of the first hello command. But popping a client
     * starts topology scanning. */
    set_apm_callbacks_pooled (&ctx, pool);
 
@@ -504,7 +504,7 @@ test_topology_discovery (void *ctx)
    replset_name = test_framework_replset_name ();
    uri_str = test_framework_get_uri_str ();
 
-   client = mongoc_client_new (uri_str);
+   client = test_framework_client_new (uri_str, NULL);
    test_framework_set_ssl_opts (client);
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
    sd_secondary = mongoc_client_select_server (client,
@@ -526,7 +526,7 @@ test_topology_discovery (void *ctx)
    uri_str_auth = test_framework_add_user_password_from_env (uri_str);
 
    mongoc_client_destroy (client);
-   client = mongoc_client_new (uri_str_auth);
+   client = test_framework_client_new (uri_str_auth, NULL);
    test_framework_set_ssl_opts (client);
    collection = get_test_collection (client, "sdam_dc_test");
    BSON_APPEND_UTF8 (&doc, "hello", "world");
@@ -567,7 +567,7 @@ test_direct_connection (void *ctx)
    replset_name = test_framework_replset_name ();
    uri_str = test_framework_get_uri_str ();
 
-   client = mongoc_client_new (uri_str);
+   client = test_framework_client_new (uri_str, NULL);
    test_framework_set_ssl_opts (client);
    mongoc_client_set_error_api (client, MONGOC_ERROR_API_VERSION_2);
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
@@ -583,20 +583,21 @@ test_direct_connection (void *ctx)
     * ``mongodb://HOST/?directConnection=true`` as the URI.
     * Attempt a write to a collection.
     *
-    * Outcome: Verify that the write failed with a NotMaster error. */
+    * Outcome: Verify that the write failed with a NotPrimary error. */
    bson_free (uri_str);
    uri_str = bson_strdup_printf ("mongodb://%s/?directConnection=true",
                                  hl_secondary->host_and_port);
    uri_str_auth = test_framework_add_user_password_from_env (uri_str);
 
    mongoc_client_destroy (client);
-   client = mongoc_client_new (uri_str_auth);
+   client = test_framework_client_new (uri_str_auth, NULL);
    test_framework_set_ssl_opts (client);
    collection = get_test_collection (client, "sdam_dc_test");
    BSON_APPEND_UTF8 (&doc, "hello", "world");
    r = mongoc_collection_insert_one (collection, &doc, NULL, &reply, &error);
    ASSERT_OR_PRINT (!r, error);
-   ASSERT (strstr (error.message, "not master"));
+   ASSERT (strstr (error.message, "not master") ||
+           strstr (error.message, "not primary"));
 
    bson_destroy (&reply);
    bson_destroy (&doc);
@@ -631,7 +632,7 @@ test_existing_behavior (void *ctx)
    replset_name = test_framework_replset_name ();
    uri_str = test_framework_get_uri_str ();
 
-   client = mongoc_client_new (uri_str);
+   client = test_framework_client_new (uri_str, NULL);
    test_framework_set_ssl_opts (client);
    mongoc_client_set_error_api (client, MONGOC_ERROR_API_VERSION_2);
    prefs = mongoc_read_prefs_new (MONGOC_READ_SECONDARY);
@@ -654,13 +655,14 @@ test_existing_behavior (void *ctx)
    uri_str_auth = test_framework_add_user_password_from_env (uri_str);
 
    mongoc_client_destroy (client);
-   client = mongoc_client_new (uri_str_auth);
+   client = test_framework_client_new (uri_str_auth, NULL);
    test_framework_set_ssl_opts (client);
    collection = get_test_collection (client, "sdam_dc_test");
    BSON_APPEND_UTF8 (&doc, "hello", "world");
    r = mongoc_collection_insert_one (collection, &doc, NULL, &reply, &error);
    ASSERT_OR_PRINT (!r, error);
-   ASSERT (strstr (error.message, "not master"));
+   ASSERT (strstr (error.message, "not master") ||
+           strstr (error.message, "not primary"));
 
    bson_destroy (&reply);
    bson_destroy (&doc);
@@ -728,7 +730,7 @@ test_prose_rtt (void *unused)
    callbacks = mongoc_apm_callbacks_new ();
    mongoc_apm_set_server_heartbeat_succeeded_cb (callbacks,
                                                  heartbeat_succeeded);
-   pool = mongoc_client_pool_new (uri);
+   pool = test_framework_client_pool_new_from_uri (uri, NULL);
    test_framework_set_pool_ssl_opts (pool);
    memset (&ctx, 0, sizeof (prose_test_ctx_t));
    mongoc_client_pool_set_apm_callbacks (pool, callbacks, &ctx);
@@ -744,7 +746,7 @@ test_prose_rtt (void *unused)
     * to succeed. */
    _mongoc_usleep (RTT_TEST_INITIAL_SLEEP_SEC * 1000 * 1000);
 
-   /* Set a failpoint to make isMaster commands take longer. */
+   /* Set a failpoint to make hello commands take longer. */
    bson_init (&cmd);
    BCON_APPEND (&cmd, "configureFailPoint", "failCommand");
    BCON_APPEND (&cmd, "mode", "{", "times", BCON_INT32 (1000), "}");
@@ -753,7 +755,8 @@ test_prose_rtt (void *unused)
                 "{",
                 "failCommands",
                 "[",
-                "isMaster",
+                HANDSHAKE_CMD_LEGACY_HELLO,
+                "hello",
                 "]",
                 "blockConnection",
                 BCON_BOOL (true),
@@ -770,9 +773,8 @@ test_prose_rtt (void *unused)
     * RTT_TEST_TIMEOUT_SEC seconds, consider it a failure. */
    satisfied = false;
    start_us = bson_get_monotonic_time ();
-   while (!satisfied &&
-          bson_get_monotonic_time () <
-             start_us + RTT_TEST_TIMEOUT_SEC * 1000 * 1000) {
+   while (!satisfied && bson_get_monotonic_time () <
+                           start_us + RTT_TEST_TIMEOUT_SEC * 1000 * 1000) {
       mongoc_server_description_t *sd;
 
       sd = mongoc_client_select_server (
